@@ -4,7 +4,7 @@
  **            Vincent Deffontaines <vincent@inl.fr>
  ** INL http://www.inl.fr/
  **
- ** $Id: internal.c 3606 2007-08-02 19:20:30Z regit $
+ ** $Id: internal.c 4193 2007-12-28 10:29:54Z regit $
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
@@ -20,8 +20,11 @@
  ** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#include "nufw_source.h"
 #include "nuclient.h"
+#include "ipv6.h"
 #include <sasl/saslutil.h>
+#include <pthread.h>
 #include <stdarg.h>		/* va_list, va_start, ... */
 #include <gnutls/x509.h>
 #include <langinfo.h>
@@ -31,6 +34,7 @@
 #include "internal.h"
 #include <sys/utsname.h>
 
+char* nu_locale_charset;
 
 /**
  * \ingroup libnuclient
@@ -69,7 +73,8 @@ static int samp_send(gnutls_session session, const char *buffer,
 	unsigned len, alloclen;
 	int result;
 
-	alloclen = ((length / 3) + 1) * 4 + 4;
+	/* prefix ("C: ") + base64 length + 1 nul byte */
+	alloclen = 3 + ((length+2)/3)*4 + 1;
 	buf = malloc(alloclen);
 	if (buf == NULL) {
 		SET_ERROR(err, INTERNAL_ERROR, MEMORY_ERR);
@@ -630,19 +635,12 @@ int init_socket(nuauth_session_t * session,
 		struct sockaddr_in6 *src6 = (struct sockaddr_in6 *)&session->src_addr;
 		if (res->ai_family == AF_INET
 		    && session->src_addr.ss_family == AF_INET6
-		    && memcmp(&src6->sin6_addr, "\0\0\0\0\0\0\0\0\0\0\xff\xff", 12) == 0)
+		    && is_ipv4(&src6->sin6_addr))
 		{
-			struct in_addr addr;
-			addr.s_addr = src6->sin6_addr.s6_addr32[3];
 			src4->sin_family = AF_INET;
-			src4->sin_addr = addr;
+			ipv6_to_ipv4(&src6->sin6_addr, &src4->sin_addr);
 		} else if (res->ai_family == AF_INET6 && session->src_addr.ss_family == AF_INET) {
-			struct in_addr addr;
-			addr.s_addr = src4->sin_addr.s_addr;
-			src6->sin6_addr.s6_addr32[0] = 0;
-			src6->sin6_addr.s6_addr32[0] = 0;
-			src6->sin6_addr.s6_addr32[2] = 0xffff0000;
-			src6->sin6_addr.s6_addr32[3] = addr.s_addr;
+			uint32_to_ipv6(src4->sin_addr.s_addr, &src6->sin6_addr);
 		} else {
 			if (session->verbose) {
 				fprintf(stderr,
@@ -687,6 +685,7 @@ int init_socket(nuauth_session_t * session,
 
 	/* connect to nuauth */
 	if (connect(session->socket, res->ai_addr, res->ai_addrlen) == -1) {
+		close(session->socket);
 		errno = ENOTCONN;
 		SET_ERROR(err, INTERNAL_ERROR, CANT_CONNECT_ERR);
 		freeaddrinfo(res);
@@ -766,7 +765,7 @@ int certificate_check(nuauth_session_t *session)
 int tls_handshake(nuauth_session_t * session, nuclient_error_t * err)
 {
 	int ret;
-	unsigned int status;
+	unsigned int status = 0;
 
 	gnutls_transport_set_ptr(session->tls,
 				 (gnutls_transport_ptr) session->socket);
@@ -807,6 +806,11 @@ int tls_handshake(nuauth_session_t * session, nuclient_error_t * err)
 					printf(", signer not found");
 				if( status & GNUTLS_CERT_SIGNER_NOT_CA )
 					printf(", signer not a CA");
+#ifdef GNUTLS_CERT_INSECURE_ALGORITHM
+				if (status & GNUTLS_CERT_INSECURE_ALGORITHM)
+					printf(", use of insecure algorithm");
+#endif
+
 				printf("\n");
 			}
 			SET_ERROR(err, GNUTLS_ERROR, GNUTLS_E_CERTIFICATE_ERROR);
@@ -877,9 +881,12 @@ void ask_session_end(nuauth_session_t * session)
 	}
 	if (session->server_mode == SRV_TYPE_PUSH) {
 		if (session->checkthread != NULL_THREAD
-		    && !pthread_equal(session->checkthread, self_thread)) {
-			pthread_cancel(session->checkthread);
+		    && !pthread_equal(session->checkthread, self_thread))
+		{
+			/* ask thread to stop */
+			(void)pthread_mutex_trylock(&session->checkthread_stop);
 			pthread_join(session->checkthread, NULL);
+			pthread_mutex_destroy(&session->checkthread_stop);
 		}
 	}
 	pthread_mutex_unlock(&(session->mutex));
