@@ -4,7 +4,7 @@
  **            Vincent Deffontaines <vincent@inl.fr>
  ** INL http://www.inl.fr/
  **
- ** $Id: libnuclient.c 3421 2007-06-08 09:05:50Z pollux $
+ ** $Id: libnuclient.c 4278 2008-01-08 17:31:23Z lds $
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
@@ -37,15 +37,19 @@
  */
 #define USE_GCRYPT_MALLOC_SECURE
 
-
+#include "nufw_source.h"
 #include "nuclient.h"
+#include <pthread.h>
+#include <sasl/sasl.h>
 #include <sasl/saslutil.h>
 #include <stdarg.h>		/* va_list, va_start, ... */
 #include <gnutls/x509.h>
+#include <gcrypt.h>
 #include <langinfo.h>
 #include <proto.h>
 #include "client.h"
 #include "security.h"
+#include "sys_config.h"
 #include "internal.h"
 #include <sys/utsname.h>
 
@@ -58,8 +62,6 @@ GCRY_THREAD_OPTION_PTHREAD_IMPL;
 #  define DH_BITS 1024
 #endif
 
-#define REQUEST_CERT 0
-
 static const int cert_type_priority[3] = { GNUTLS_CRT_X509, 0 };
 
 
@@ -81,9 +83,6 @@ void nu_exit_clean(nuauth_session_t * session)
 		gnutls_certificate_free_keys(session->cred);
 		gnutls_certificate_free_credentials(session->cred);
 	}
-	if (session->diffie_hellman) {
-		gnutls_dh_params_deinit(session->dh_params);
-	}
 	gnutls_deinit(session->tls);
 
 	pthread_cond_destroy(&(session->check_cond));
@@ -226,7 +225,21 @@ char *nu_get_home_dir()
 	return dir;
 }
 
+int check_key_perms(const char* filename)
+{
+	struct stat infos;
 
+	if (stat(filename, &infos) != 0) {
+		return 0;
+	}
+	
+	/* Check it's owned by the current user */
+	if (infos.st_mode & S_IRGRP || infos.st_mode & S_IROTH) {
+		return 0;
+	}
+	
+	return 1;
+}
 
 /**
  * \ingroup nuclientAPI
@@ -253,11 +266,19 @@ int nu_client_setup_tls(nuauth_session_t * session,
 	char certstring[256];
 	char keystring[256];
 	char *home = nu_get_home_dir();
+	int exit_on_error = 0;
 	int ok;
 	int ret;
 
 	session->tls_password = tls_password;
 
+	/* If the user specified a certficate and a key on command line,
+	 * exit if we fail loading them.
+	 * Elsewise, try loading certs from ~/.nufw/, but continue if we fail
+	 */
+	if (certfile || keyfile)
+		exit_on_error = 1;
+
 	/* compute patch keyfile */
 	if (keyfile == NULL && home != NULL) {
 		ok = secure_snprintf(keystring, sizeof(keystring),
@@ -268,15 +289,16 @@ int nu_client_setup_tls(nuauth_session_t * session,
 
 	/* test if key file exists */
 	if (keyfile != NULL && access(keyfile, R_OK) != 0) {
-#if REQUEST_CERT
-		printf("Unable ot load key file : %s\n", keyfile);
-		SET_ERROR(err, INTERNAL_ERROR, FILE_ACCESS_ERR);
-		if (home) {
-			free(home);
+		if (exit_on_error) {
+			if (session->verbose)
+			    printf("Unable to load key file: %s\n", keyfile);
+			SET_ERROR(err, INTERNAL_ERROR, FILE_ACCESS_ERR);
+			if (home) {
+				free(home);
+			}
+			errno = EBADF;
+			return 0;
 		}
-		errno = EBADF;
-		return 0;
-#endif
 		keyfile = NULL;
 	}
 
@@ -288,15 +310,15 @@ int nu_client_setup_tls(nuauth_session_t * session,
 	}
 	/* test if cert exists */
 	if (certfile != NULL && access(certfile, R_OK) != 0) {
-#if REQUEST_CERT
-		printf("Unable ot load certificate file : %s\n", certfile);
-		SET_ERROR(err, INTERNAL_ERROR, FILE_ACCESS_ERR);
-		if (home) {
-			free(home);
+		if (exit_on_error) {
+			printf("Unable to load certificate file : %s\n", certfile);
+			SET_ERROR(err, INTERNAL_ERROR, FILE_ACCESS_ERR);
+			if (home) {
+				free(home);
+			}
+			errno = EBADF;
+			return 0;
 		}
-		errno = EBADF;
-		return 0;
-#endif
 		certfile = NULL;
 	}
 	if (cafile == NULL && home != NULL) {
@@ -304,18 +326,25 @@ int nu_client_setup_tls(nuauth_session_t * session,
 				     "%s/.nufw/cacert.pem", home);
 		if (ok)
 			cafile = castring;
+		/* if computed cafile is not present we reset to cafile to NULL */
+		if (access(cafile, R_OK) != 0) {
+			if (errno == EACCES) {
+				printf("Warning, unable to access CA file : %s\n", cafile);
+			}
+			cafile = NULL;
+		}
 	}
 	/* test if cert exists */
 	if (cafile != NULL && access(cafile, R_OK) != 0) {
-#if REQUEST_CERT
-		printf("Unable to load CA file : %s\n", cafile);
-		SET_ERROR(err, INTERNAL_ERROR, FILE_ACCESS_ERR);
-		errno = EBADF;
-		if (home) {
-			free(home);
+		if (exit_on_error) {
+			printf("Unable to load CA file : %s\n", cafile);
+			SET_ERROR(err, INTERNAL_ERROR, FILE_ACCESS_ERR);
+			errno = EBADF;
+			if (home) {
+				free(home);
+			}
+			return 0;
 		}
-		return 0;
-#endif
 		cafile = NULL;
 	}
 
@@ -331,6 +360,12 @@ int nu_client_setup_tls(nuauth_session_t * session,
 		}
 		session->need_ca_verif = 1;
 	}
+ 
+ 	if (keyfile && !check_key_perms(keyfile)) {
+		printf("Private key have too open permissions.\n");
+		SET_ERROR(err, INTERNAL_ERROR, FILE_ACCESS_ERR);
+		return 0;
+	}
 
 	if (certfile != NULL && keyfile != NULL) {
 		ret =
@@ -420,7 +455,6 @@ int nu_client_reset_tls(nuauth_session_t *session)
 /**
  * \brief Init connection to nuauth server
  *
- * \param diffie_hellman If equals to 1, use Diffie Hellman for key exchange
  * (very secure but initialization is slower)
  * \param err Pointer to a nuclient_error_t: which contains the error
  * \return A pointer to a valid ::nuauth_session_t structure or NULL if init has failed
@@ -433,7 +467,7 @@ int nu_client_reset_tls(nuauth_session_t *session)
  *
  * If everything is ok, create the connection table using tcptable_init().
  */
-nuauth_session_t *_nu_client_new(unsigned char diffie_hellman, nuclient_error_t * err)
+nuauth_session_t *_nu_client_new(nuclient_error_t * err)
 {
 	conntable_t *new;
 	nuauth_session_t *session;
@@ -453,7 +487,6 @@ nuauth_session_t *_nu_client_new(unsigned char diffie_hellman, nuclient_error_t
 	/* Set basic fields */
 	session->userid = getuid();
 	session->connected = 0;
-	session->diffie_hellman = diffie_hellman;
 	session->count_msg_cond = -1;
 	session->auth_by_default = 1;
 	session->packet_seq = 0;
@@ -467,6 +500,10 @@ nuauth_session_t *_nu_client_new(unsigned char diffie_hellman, nuclient_error_t
 	session->timestamp_last_sent = time(NULL);
 	session->need_set_cred = 1;
 	session->need_ca_verif = 0;
+	session->default_hostname = NULL;
+	session->default_port = NULL;
+
+	load_sys_config(session);
 
 	/* create session mutex */
 	pthread_mutex_init(&(session->mutex), NULL);
@@ -488,33 +525,6 @@ nuauth_session_t *_nu_client_new(unsigned char diffie_hellman, nuclient_error_t
 		return NULL;
 	}
 
-	if (session->diffie_hellman) {
-		/* allocate diffie hellman parameters */
-		ret = gnutls_dh_params_init(&session->dh_params);
-		if (ret < 0) {
-			SET_ERROR(err, GNUTLS_ERROR, ret);
-			nu_exit_clean(session);
-			return NULL;
-		}
-
-		/* Generate Diffie Hellman parameters - for use with DHE
-		 * kx algorithms. These should be discarded and regenerated
-		 * once a day, once a week or once a month. Depending on the
-		 * security requirements.
-		 */
-		ret =
-		    gnutls_dh_params_generate2(session->dh_params,
-					       DH_BITS);
-		if (ret < 0) {
-			SET_ERROR(err, GNUTLS_ERROR, ret);
-			nu_exit_clean(session);
-			return NULL;
-		}
-
-		gnutls_certificate_set_dh_params(session->cred,
-						 session->dh_params);
-	}
-
 	if (!nu_client_reset_tls(session))
 	{
 		SET_ERROR(err, GNUTLS_ERROR, ret);
@@ -540,7 +550,7 @@ nuauth_session_t *_nu_client_new(unsigned char diffie_hellman, nuclient_error_t
 
 nuauth_session_t *nu_client_new_callback(void *username_callback,
 		      void *passwd_callback,
-		      unsigned char diffie_hellman, nuclient_error_t * err)
+		      /* TODO: To remove */ unsigned char diffie_hellman, nuclient_error_t * err)
 {
 	nuauth_session_t *session = NULL;
 
@@ -549,7 +559,7 @@ nuauth_session_t *nu_client_new_callback(void *username_callback,
 		return NULL;
 	}
 
-	session = _nu_client_new(diffie_hellman, err);
+	session = _nu_client_new(err);
 
 	session->username_callback = username_callback;
 	session->passwd_callback = passwd_callback;
@@ -574,7 +584,7 @@ nuauth_session_t *nu_client_new_callback(void *username_callback,
 
 nuauth_session_t *nu_client_new(const char *username,
 		      const char *password,
-		      unsigned char diffie_hellman, nuclient_error_t * err)
+		      /* TODO: To remove */ unsigned char diffie_hellman, nuclient_error_t * err)
 {
 	nuauth_session_t *session = NULL;
 
@@ -583,7 +593,7 @@ nuauth_session_t *nu_client_new(const char *username,
 		return NULL;
 	}
 
-	session = _nu_client_new(diffie_hellman, err);
+	session = _nu_client_new(err);
 
 	session->username = secure_str_copy(username);
 	session->password = secure_str_copy(password);
