@@ -4,7 +4,7 @@
  **            Vincent Deffontaines <vincent@inl.fr>
  ** INL http://www.inl.fr
  **
- ** $Id: tcptable.c 2795 2007-02-26 08:00:17Z regit $
+ ** $Id: tcptable.c 4121 2007-12-18 13:07:50Z haypo $
  **
  **
  ** This program is free software; you can redistribute it and/or modify
@@ -20,10 +20,13 @@
  ** along with this program; if not, write to the Free Software
  ** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
-#include "nuclient.h"
 #include "client.h"
+#include "nuclient.h"
 #include "libnuclient.h"
-#include <proto.h>
+#include "proto.h"
+#include <pthread.h>
+#include <inttypes.h>
+#include <ipv6.h>
 #define USE_JHASH3
 #include <jhash.h>
 #ifdef FREEBSD
@@ -39,6 +42,7 @@
 #include <netinet/tcp_fsm.h>
 #include <netinet/in_pcb.h>
 #include <netinet/tcp_var.h>
+
 #endif
 
 /**
@@ -55,34 +59,6 @@
 
 #ifdef LINUX
 
-/**
- * Convert an IPv6 address in "Linux format" (with no ":") into
- * in6_addr structure.
- *
- * \return Returns 0 if fails, 1 otherwise
- */
-int str2ipv6(char *text, struct in6_addr *ip)
-{
-	if (sscanf
-	    (text + 8 * 3, "%08lx",
-	     (unsigned long *) &ip->s6_addr32[3]) != 1)
-		return 0;
-	text[8 * 3] = 0;
-	if (sscanf
-	    (text + 8 * 2, "%08lx",
-	     (unsigned long *) &ip->s6_addr32[2]) != 1)
-		return 0;
-	text[8 * 2] = 0;
-	if (sscanf(text + 8, "%08lx", (unsigned long *) &ip->s6_addr32[1])
-	    != 1)
-		return 0;
-	text[8] = 0;
-	if (sscanf(text, "%08lx", (unsigned long *) &ip->s6_addr32[0]) !=
-	    1)
-		return 0;
-	return 1;
-}
-
 /**
  * Parse a Linux connection table (/proc/net/tcp or /proc/net/udp) and filter
  * connection: only keep session user connections in state "SYN packet sent".
@@ -155,26 +131,22 @@ int parse_tcptable_file(nuauth_session_t * session, conntable_t * ct, char *file
 
 		/* get all fields */
 		if (!use_ipv6) {
-			c.ip_src.s6_addr32[0] = 0;
-			c.ip_src.s6_addr32[1] = 0;
-			c.ip_src.s6_addr32[2] = 0xffff0000;
-			c.ip_dst.s6_addr32[0] = 0;
-			c.ip_dst.s6_addr32[1] = 0;
-			c.ip_dst.s6_addr32[2] = 0xffff0000;
+			uint32_t src, dst;
 			ret = sscanf(buf,
 				     "%*d: "
-				     "%lx:%hx "
-				     "%lx:%hx "
+				     "%" SCNx32 ":%hx "
+				     "%" SCNx32 ":%hx "
 				     "%*x %*x:%*x %*x:%*x %x "
 				     "%lu %*d %lu",
-				     (long *) &c.ip_src.s6_addr32[3],
-				     &c.port_src,
-				     (long *) &c.ip_dst.s6_addr32[3],
-				     &c.port_dst, &c.retransmit, &c.uid,
+				     &src, &c.port_src,
+				     &dst, &c.port_dst,
+				     &c.retransmit, &c.uid,
 				     &c.inode);
 			if (ret != 7) {
 				continue;
 			}
+			uint32_to_ipv6(src, &c.ip_src);
+			uint32_to_ipv6(dst, &c.ip_dst);
 		} else {
 			char ip_src[33];
 			char ip_dst[33];
@@ -194,9 +166,9 @@ int parse_tcptable_file(nuauth_session_t * session, conntable_t * ct, char *file
 			if (ret != 7) {
 				continue;
 			}
-			if (!str2ipv6(ip_src, &c.ip_src))
+			if (!hex2ipv6(ip_src, &c.ip_src))
 				continue;
-			if (!str2ipv6(ip_dst, &c.ip_dst))
+			if (!hex2ipv6(ip_dst, &c.ip_dst))
 				continue;
 		}
 
@@ -342,12 +314,12 @@ int tcptable_read(nuauth_session_t * session, conntable_t * ct)
 		    || (inet_lnaof(inp->inp_laddr) == INADDR_ANY))
 			continue;
 
-		c.lcl = inp->inp_laddr.s_addr;
-		c.lclp = inp->inp_lport;
+		uint32_to_ipv6(inp->inp_laddr.s_addr, &c.ip_src);
+		c.port_src = inp->inp_lport;
 
-		c.rmt = inp->inp_faddr.s_addr;
-		c.rmtp = inp->inp_fport;
-		c.proto = IPPROTO_TCP;
+		uint32_to_ipv6(inp->inp_faddr.s_addr, &c.ip_dst);
+		c.port_dst = inp->inp_fport;
+		c.protocol = IPPROTO_TCP;
 
 		tcptable_add(ct, &c);
 	}
@@ -384,10 +356,17 @@ int tcptable_init(conntable_t ** ct)
 inline int tcptable_hash(conn_t * c)
 {
 	/* TODO: Hash the whole ip address! */
+#ifndef FREEBSD
 	return (jhash_3words(c->ip_src.s6_addr32[3],
 			     c->ip_dst.s6_addr32[3],
 			     (c->port_dst | c->port_src << 16),
 			     32)) % CONNTABLE_BUCKETS;
+#else
+	return (jhash_3words(c->ip_src.__u6_addr.__u6_addr32[3],
+			     c->ip_dst.__u6_addr.__u6_addr32[3],
+			     (c->port_dst | c->port_src << 16),
+			     32)) % CONNTABLE_BUCKETS;
+#endif
 }
 
 /**
@@ -430,11 +409,9 @@ conn_t *tcptable_find(conntable_t * ct, conn_t * c)
 	bucket = ct->buckets[tcptable_hash(c)];
 	while (bucket != NULL) {
 		if ((c->protocol == bucket->protocol)
-		    && memcmp(&c->ip_dst, &bucket->ip_dst,
-			      sizeof(c->ip_dst)) == 0
+		    && ipv6_equal(&c->ip_dst, &bucket->ip_dst)
 		    && (c->port_dst == bucket->port_dst)
-		    && memcmp(&c->ip_src, &bucket->ip_src,
-			      sizeof(c->ip_src)) == 0
+		    && ipv6_equal(&c->ip_src, &bucket->ip_src)
 		    && (c->port_src == bucket->port_src)
 		    ) {
 			return bucket;
