@@ -4,7 +4,7 @@
  **            Vincent Deffontaines <vincent@inl.fr>
  ** INL http://www.inl.fr/
  **
- ** $Id: nutcpc.c 3547 2007-07-06 09:30:35Z pollux $
+ ** $Id: nutcpc.c 4039 2007-12-05 16:39:02Z lds $
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
@@ -20,6 +20,8 @@
  ** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#include "nufw_source.h"
+
 #include "../lib/nuclient.h"
 #include <locale.h>
 #include <config.h>
@@ -29,7 +31,7 @@
 #include <signal.h>
 #include "proto.h"
 #include "security.h"
-#define NUTCPC_VERSION PACKAGE_VERSION " $Revision: 3547 $"
+#define NUTCPC_VERSION PACKAGE_VERSION " $Revision: 4039 $"
 
 #ifdef FREEBSD
 #include <readpassphrase.h>
@@ -87,7 +89,10 @@ char *compute_run_pid()
 		return NULL;
 	secure_snprintf(path_dir, sizeof(path_dir), "%s/.nufw", home);
 	if (access(path_dir, R_OK) != 0) {
-		mkdir(path_dir, S_IRWXU);
+		printf("Creating directory \"%s\"\n", path_dir);
+		if (mkdir(path_dir, S_IRWXU) != 0) {
+			printf("Could not create directory \"%s\" (%s)\n", path_dir, strerror(errno));
+		}
 	}
 	secure_snprintf(path_dir, sizeof(path_dir), "%s/.nufw/nutcpc", home);
 	free(home);
@@ -323,7 +328,7 @@ static void usage(void)
 	fprintf(stderr, "  -V: display version\n");
 	fprintf(stderr, "\n");
 	fprintf(stderr, "Certificate options:\n");
-	fprintf(stderr, "  -C CERTFILE: certificate filanem\n");
+	fprintf(stderr, "  -C CERTFILE: certificate filename\n");
 	fprintf(stderr, "  -A AUTHFILE: authority certificate filename\n");
 	fprintf(stderr, "  -K KEYFILE:  key filename\n");
 	fprintf(stderr, "  -W CERTPASS: certificate password\n");
@@ -458,6 +463,27 @@ nuauth_session_t *do_connect(nutcpc_context_t * context, char *username)
 
 	nu_client_set_debug(session, context->debug_mode);
 
+	/* Set hostname from libnuclient if it wasn't specified by the user */
+	if(*context->port == '\0')
+	{
+		if(session->default_port)
+			SECURE_STRNCPY(context->port, session->default_port,
+			       sizeof(context->port));
+		else
+			SECURE_STRNCPY(context->port, USERPCKT_SERVICE,
+				       sizeof(context->port));
+	}
+
+	if(*context->srv_addr == '\0')
+	{
+		if(session->default_hostname)
+			SECURE_STRNCPY(context->srv_addr, session->default_hostname,
+			       sizeof(context->srv_addr));
+		else
+			SECURE_STRNCPY(context->srv_addr, NUAUTH_IP,
+				       sizeof(context->srv_addr));
+	}
+
 	if (!nu_client_setup_tls(session, context->keyfile, context->certfile,
 	     context->cafile, context->cert_password, err)) {
 		nu_client_delete(session);
@@ -563,10 +589,6 @@ void parse_cmdline_options(int argc, char **argv,
 	int stealth = 0;
 
 	/* set default values */
-	SECURE_STRNCPY(context->port, USERPCKT_SERVICE,
-		       sizeof(context->port));
-	SECURE_STRNCPY(context->srv_addr, NUAUTH_IP,
-		       sizeof(context->srv_addr));
 	context->interval = 100;
 	context->donotuselock = 0;
 	context->debug_mode = 0;
@@ -683,6 +705,7 @@ void init_library(nutcpc_context_t * context, char *username)
 	if (session == NULL) {
 		printf("Unable to initiate connection to NuFW gateway\n");
 		printf("Problem: %s\n", nu_client_strerror(err));
+		printf("Authentication failed (check parameters)\n");
 		exit(EXIT_FAILURE);
 	}
 }
