@@ -3,7 +3,7 @@
  ** Written by Eric Leblond <regit@inl.fr>
  ** INL http://www.inl.fr/
  **
- ** $Id: nufw_source.h 2738 2007-02-17 13:59:56Z regit $
+ ** $Id: nufw_source.h 3793 2007-10-08 16:48:27Z regit $
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
@@ -22,11 +22,15 @@
 #ifndef NUFW_SOURCE_H
 #define NUFW_SOURCE_H
 
+#ifdef _FEATURES_H
+#   error "nufw_source.h have to be included before <features.h>"
+#endif
+
 /**
  * Use POSIX standard, version "IEEE 1003.1-2004",
  * Neded by sigaction (signal.h) and timespec (time.h) for example
  */
-#ifdef __linux__
+#if defined(__linux__)
 #  define _POSIX_C_SOURCE 199506L
 #endif
 
@@ -41,6 +45,12 @@
  */
 #define _BSD_SOURCE
 
+/**
+ * Use SVID standard,
+ * needed to get 'strdup' from <string.h>.
+ */
+#define _SVID_SOURCE
+
 /* Disable inline keyword when compiling in strict ANSI conformance */
 #if defined(__STRICT_ANSI__) && !defined(__cplusplus)
 #  undef inline
