@@ -1,7 +1,7 @@
 /*
-** Copyright(C) 2003-2005 Eric Leblond <regit@inl.fr>
+** Copyright(C) 2003-2007 Eric Leblond <regit@inl.fr>
 **
-** $Id: auth_srv.h 3675 2007-08-20 15:23:54Z regit $
+** $Id: auth_srv.h 4056 2007-12-08 11:07:02Z regit $
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -43,6 +43,7 @@
 #include <unistd.h>
 #include <netdb.h>
 #include <sys/types.h>
+#include <sys/stat.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
@@ -118,7 +119,6 @@ typedef enum {
 #define NUAUTH_TLS_MAX_SERVERS 16
 #define TLS_CLIENT_MIN_DELAY 25000
 #define AUTH_NEGO_TIMEOUT 30
-#define LOG_USERS_LEVEL 9
 
 #define UNKNOWN_STRING "UNKNOWN"
 
@@ -142,11 +142,12 @@ typedef enum {
  * option "nuauth_session_duration". See member session_duration of ::nuauth_params.
  */
 #define SESSION_DURATION 0
+#define DEFAULT_PROTO_WAIT_DELAY 2
 #define DEFAULT_USERAUTH_MODULE "libplaintext"
 #define DEFAULT_ACLS_MODULE "libplaintext"
 #define DEFAULT_PERIODS_MODULE "libxml_defs"
 #define DEFAULT_LOGS_MODULE "libsyslog"
-#define DEFAULT_IPAUTH_MODULE "libident"
+#define DEFAULT_IPAUTH_MODULE "ipauth_guest"
 #define DEFAULT_CERTIFICATE_CHECK_MODULE "libx509_std"
 #define DEFAULT_CERTIFICATE_TO_UID_MODULE "libx509_std"
 #define DEFAULT_USER_SESSION_MODIFY_MODULE "libsession_expire"
@@ -222,13 +223,16 @@ typedef enum {
 /**
  * Nuauth full version, eg. "nuauth 2.1.2 (Revision: 2730)"
  */
-#define NUAUTH_FULL_VERSION (VERSION " ($Revision: 3675 $)")
+#define NUAUTH_FULL_VERSION (VERSION " ($Revision: 4056 $)")
 
 void nuauth_ask_exit();
-void stop_pool_threads(gboolean wait);
-void block_pool_threads();
-void start_pool_threads();
-void release_pool_threads();
+void stop_all_thread_pools(gboolean wait);
+void block_thread_pools();
+void release_thread_pools();
+void start_all_thread_pools();
+void stop_thread_pool(const char *name, GThreadPool **pool);
 void nuauth_install_signals(gboolean action);
 
+int nuauth_bind(char **errmsg, const char *addr, const char *port, char *context);
+
 #endif
