@@ -2,7 +2,7 @@
  ** Copyright(C) 2004-2007 INL
  ** Written by Eric Leblond <regit@inl.fr>
  **
- ** $Id: authsrv.c 3616 2007-08-09 23:20:12Z gryzor $
+ ** $Id: authsrv.c 4128 2007-12-18 13:23:22Z haypo $
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
@@ -71,12 +71,12 @@ void cleanup_func_remove(cleanup_func_t func)
 	cleanup_func_list = g_list_remove(cleanup_func_list, func);
 }
 
-void block_pool_threads()
+void block_thread_pools()
 {
 	nuauthdatas->need_reload = 1;
 }
 
-void release_pool_threads()
+void release_thread_pools()
 {
 	/* liberate threads by broadcasting condition */
 	nuauthdatas->need_reload = 0;
@@ -86,7 +86,7 @@ void release_pool_threads()
 	g_mutex_unlock(nuauthdatas->reload_cond_mutex);
 }
 
-void start_pool_threads()
+void start_all_thread_pools()
 {
 	if (nuauthconf->do_ip_authentication) {
 		/* create thread of pool */
@@ -136,34 +136,37 @@ void start_pool_threads()
 	}
 }
 
-void stop_pool_threads(gboolean wait)
+/**
+ * Stop one thread pool
+ */
+void stop_thread_pool(const char *name, GThreadPool **pool)
+{
+	log_message(DEBUG, DEBUG_AREA_MAIN,
+			"Stop thread pool '%s'", name);
+	g_thread_pool_free(*pool, TRUE, TRUE);
+	*pool = NULL;
+}
+
+/**
+ * Stop all nuauth thread pools
+ * If soft is true, destroy pools.
+ */
+void stop_all_thread_pools(gboolean soft)
 {
 	/* end logging threads */
-	if (wait) {
-		log_message(DEBUG, DEBUG_AREA_MAIN,
-			    "Stop thread pool 'user session loggers'");
-		g_thread_pool_free(nuauthdatas->user_session_loggers, TRUE,
-				   wait);
-		log_message(DEBUG, DEBUG_AREA_MAIN,
-			    "Stop thread pool 'user loggers'");
-		g_thread_pool_free(nuauthdatas->user_loggers, TRUE, wait);
-		log_message(DEBUG, DEBUG_AREA_MAIN,
-			    "Stop thread pool 'acl checkers'");
-		g_thread_pool_free(nuauthdatas->acl_checkers, TRUE, wait);
+	if (soft) {
+		stop_thread_pool("session logger", &nuauthdatas->user_session_loggers);
+		stop_thread_pool("packet logger", &nuauthdatas->user_loggers);
+		stop_thread_pool("acl checker", &nuauthdatas->acl_checkers);
 
 		if (nuauthconf->log_users_sync) {
-			log_message(DEBUG, DEBUG_AREA_MAIN,
-				    "Stop thread pool 'decision workers'");
-			g_thread_pool_free(nuauthdatas->decisions_workers,
-					   TRUE, wait);
+			stop_thread_pool("decision worker",
+					&nuauthdatas->decisions_workers);
 		}
 
 		if (nuauthconf->do_ip_authentication) {
-			log_message(DEBUG, DEBUG_AREA_MAIN,
-				    "Stop thread pool 'ip auth workers'");
-			g_thread_pool_free(nuauthdatas->
-					   ip_authentication_workers, TRUE,
-					   wait);
+			stop_thread_pool("ip auth worker",
+					&nuauthdatas->ip_authentication_workers);
 		}
 	}
 	g_thread_pool_stop_unused_threads();
@@ -203,12 +206,12 @@ void stop_threads(gboolean wait)
 	}
 
 	/* kill entries point */
-	thread_stop(&nuauthdatas->tls_auth_server);
-	thread_stop(&nuauthdatas->tls_nufw_server);
+	thread_list_stop(nuauthdatas->tls_auth_servers);
+	thread_list_stop(nuauthdatas->tls_nufw_servers);
 	thread_stop(&nuauthdatas->pre_client_thread);
 	if (wait) {
-		thread_wait_end(&nuauthdatas->tls_auth_server);
-		thread_wait_end(&nuauthdatas->tls_nufw_server);
+		thread_list_wait_end(nuauthdatas->tls_auth_servers);
+		thread_list_wait_end(nuauthdatas->tls_nufw_servers);
 		thread_wait_end(&nuauthdatas->pre_client_thread);
 	}
 
@@ -233,7 +236,7 @@ void stop_threads(gboolean wait)
 		thread_wait_end(&nuauthdatas->localid_auth_thread);
 	}
 
-	stop_pool_threads(wait);
+	stop_all_thread_pools(wait);
 	/* done! */
 	log_message(INFO, DEBUG_AREA_MAIN, "Threads stopped.");
 }
@@ -243,8 +246,8 @@ void free_threads()
 	/* free all thread mutex */
 	thread_destroy(&nuauthdatas->tls_pusher);
 	thread_destroy(&nuauthdatas->search_and_fill_worker);
-	thread_destroy(&nuauthdatas->tls_auth_server);
-	thread_destroy(&nuauthdatas->tls_nufw_server);
+	thread_list_destroy(nuauthdatas->tls_auth_servers);
+	thread_list_destroy(nuauthdatas->tls_nufw_servers);
 	thread_destroy(&nuauthdatas->limited_connections_handler);
 	if (nuauthconf->push && nuauthconf->hello_authentication) {
 		thread_destroy(&nuauthdatas->localid_auth_thread);
@@ -310,6 +313,10 @@ void nuauth_deinit(gboolean soft)
 	free_threads();
 	clear_push_queue();
 
+	g_hash_table_destroy(conn_list);
+
+	g_static_mutex_free(&insert_mutex);
+
 	/* destroy pid file */
 	unlink(NUAUTH_PID_FILE);
 }
@@ -439,6 +446,7 @@ void print_usage()
 		"\t-D : run as a daemon, send debug messages to syslog (else stdout/stderr)\n"
 		"\t-V : display version and exit\n"
 		"\t-v : increase debug level (+1 for each 'v') (max useful number : 10)\n"
+		"\t-p : specify listening TCP port (this port waits for nufw) (default : 4128)\n"
 		"\t-l : specify listening TCP port (this port waits for clients) (default : 4129)\n"
 		"\t-L : specify NUFW listening IP address (local) (this address waits for nufw data) (default : 127.0.0.1)\n"
 		"\t-C : specify clients listening IP address (local) (this address waits for clients auth) (default : 0.0.0.0)\n"
@@ -469,37 +477,32 @@ void parse_options(int argc, char **argv, command_line_params_t * params)
 
 		case 'l':
 			/* port we listen for auth answer */
+			g_free(nuauthconf->userpckt_port);
+			nuauthconf->userpckt_port = g_strdup(optarg);
 			printf("Waiting for user packets on TCP port %s\n",
-			       optarg);
-			SECURE_STRNCPY(nuauthconf->userpckt_port, optarg,
-				       sizeof(nuauthconf->userpckt_port));
+			       nuauthconf->userpckt_port);
+			break;
+
+		case 'p':
+			/* port we listen for auth answer */
+			g_free(nuauthconf->authreq_port);
+			nuauthconf->authreq_port = g_strdup(optarg);
+			printf("Waiting for nufw packets on TCP port %s\n",
+			       nuauthconf->authreq_port);
 			break;
 
 		case 'L':
-			/* Adress we listen for NUFW originating packets */
-			/* SECURE_STRNCPY(nufw_listen_address, optarg, HOSTNAME_SIZE); */
-			params->nuauth_nufw_listen_addr =
-			    (char *) calloc(HOSTNAME_SIZE, sizeof(char));
-			if (params->nuauth_nufw_listen_addr == NULL) {
-				/* TODO: Error message and free memory? */
-				exit(EXIT_FAILURE);
-			}
-			SECURE_STRNCPY(params->nuauth_nufw_listen_addr,
-				       optarg, HOSTNAME_SIZE);
+			/* Address we listen on for NUFW originating packets */
+			g_free(params->nuauth_nufw_listen_addr);
+			params->nuauth_nufw_listen_addr = g_strdup(optarg);
 			printf("Waiting for Nufw daemon packets on %s\n",
 			       params->nuauth_nufw_listen_addr);
 			break;
 
 		case 'C':
-			/* Adress we listen for NUFW originating packets */
-			params->nuauth_client_listen_addr =
-			    (char *) calloc(HOSTNAME_SIZE, sizeof(char));
-			if (params->nuauth_client_listen_addr == NULL) {
-				/* TODO: Error message and free memory? */
-				exit(EXIT_FAILURE);
-			}
-			SECURE_STRNCPY(params->nuauth_client_listen_addr,
-				       optarg, HOSTNAME_SIZE);
+			/* Address we listen on for client originating packets */
+			g_free(params->nuauth_client_listen_addr);
+			params->nuauth_client_listen_addr = g_strdup(optarg);
 			printf("Waiting for clients auth packets on %s\n",
 			       params->nuauth_client_listen_addr);
 			break;
@@ -583,12 +586,12 @@ void no_action_signals(int recv_signal)
 		case SIGINT:
 		log_message(CRITICAL, DEBUG_AREA_MAIN,
 			    "[+] Nuauth received SIGINT (leaving)");
-		exit(-1);
+		exit(EXIT_FAILURE);
 		break;
 		case SIGTERM:
 		log_message(CRITICAL, DEBUG_AREA_MAIN,
 				"[+] Nuauth received SIGTERM (leaving)");
-		exit(-1);
+		exit(EXIT_FAILURE);
 		break;
 		case SIGHUP:
 		log_message(CRITICAL, DEBUG_AREA_MAIN,
@@ -616,7 +619,7 @@ void configure_app(int argc, char **argv)
 	/* Avoid creation of core file which may contains username and password */
 	if (getrlimit(RLIMIT_CORE, &core_limit) == 0) {
 #ifdef DEBUG_ENABLE
-		core_limit.rlim_cur = -1;
+		core_limit.rlim_cur = RLIM_INFINITY;
 #else
 		core_limit.rlim_cur = 0;
 #endif
@@ -650,7 +653,10 @@ void configure_app(int argc, char **argv)
 	params.nuauth_nufw_listen_addr = NULL;
 
 	/* load configuration */
-	init_nuauthconf(&nuauthconf);
+	if (!init_nuauthconf(&nuauthconf)) {
+		fprintf(stderr,"Unable to load configuration\n");
+		exit(EXIT_FAILURE);
+	}
 
 	log_message(INFO, DEBUG_AREA_MAIN, "Start NuAuth server.");
 
@@ -671,7 +677,7 @@ void configure_app(int argc, char **argv)
 
 	parse_options(argc, argv, &params);
 
-	build_prenuauthconf(nuauthconf, NULL);
+	build_prenuauthconf(nuauthconf, NULL, 0);
 
 	if (nuauthconf->uses_utf8) {
 		setlocale(LC_ALL, "");
@@ -686,13 +692,17 @@ void configure_app(int argc, char **argv)
 		    nuauthconf->debug_level);
 
 	/* init credential */
-	create_x509_credentials();
+	if(! create_x509_credentials())
+	{
+		fprintf(stderr,"Certificate initialization failed\n");
+		exit(EXIT_FAILURE);
+	}
 
 	if (params.daemonize == 1) {
 		daemonize();
 	} else {
 		g_message("[+] Starting nuauth " VERSION
-			  " ($Revision: 3616 $) with config "
+			  " ($Revision: 4128 $) with config "
 			  DEFAULT_CONF_FILE );
 	}
 }
@@ -718,8 +728,8 @@ void configure_app(int argc, char **argv)
  *      - tls_pusher with push_worker() ;
  *      - search_and_fill_worker with search_and_fill() ;
  *      - localid_auth_thread with localid_auth() (if needed) ;
- *      - tls_auth_server with tls_user_authsrv() ;
- *      - tls_nufw_server with tls_nufw_authsrv() ;
+ *      - tls_auth_servers with tls_user_start_servers() ;
+ *      - tls_nufw_servers with tls_nufw_start_servers() ;
  *      - limited_connections_handler with limited_connection_handler().
  *
  * Other queue, threads, etc. are created elsewhere:
@@ -727,14 +737,14 @@ void configure_app(int argc, char **argv)
  */
 void init_nuauthdatas()
 {
-	block_pool_threads();
+	block_thread_pools();
 	nuauthdatas->tls_push_queue = g_async_queue_new();
 	if (!nuauthdatas->tls_push_queue)
 		exit(EXIT_FAILURE);
 
 	/* initialize packets list */
 	conn_list = g_hash_table_new_full((GHashFunc) hash_connection,
-					  compare_connection,
+					  (GEqualFunc) tracking_equal,
 					  NULL, (GDestroyNotify)
 					  free_connection);
 
@@ -745,7 +755,8 @@ void init_nuauthdatas()
 
 	/* init and load modules */
 	init_modules_system();
-	load_modules();
+	if(!load_modules())
+		exit(EXIT_FAILURE);
 
 	/* init periods */
 	nuauthconf->periods = init_periods(nuauthconf);
@@ -757,7 +768,7 @@ void init_nuauthdatas()
 	if (nuauthconf->user_cache)
 		init_user_cache();
 
-	start_pool_threads();
+	start_all_thread_pools();
 
 	null_message = g_new0(struct cache_message, 1);
 	null_queue_datas = g_new0(gchar, 1);
@@ -768,6 +779,7 @@ void init_nuauthdatas()
 	nuauthdatas->userqueue =
 	    g_private_new((GDestroyNotify) g_async_queue_unref);
 
+	g_static_mutex_init(&insert_mutex);
 	/* create thread for search_and_fill thread */
 	log_message(VERBOSE_DEBUG, DEBUG_AREA_MAIN,
 		    "Creating search_and_fill thread");
@@ -800,17 +812,16 @@ void init_nuauthdatas()
 
 	/* create TLS authentication server threads (auth + nufw) */
 	log_message(VERBOSE_DEBUG, DEBUG_AREA_MAIN,
-		    "Creating tls authentication server thread");
-	thread_new(&nuauthdatas->tls_auth_server, "tls auth server",
-		      tls_user_authsrv);
+		    "Creating tls authentication server threads");
+	tls_user_start_servers(nuauthdatas->tls_auth_servers);
 
 	log_message(VERBOSE_DEBUG, DEBUG_AREA_MAIN,
-		    "Creating tls nufw server thread");
-	thread_new(&nuauthdatas->tls_nufw_server, "tls nufw server",
-		      tls_nufw_authsrv);
+		    "Creating tls nufw server threads");
+
+	tls_nufw_start_servers(nuauthdatas->tls_nufw_servers);
 
 	log_message(INFO, DEBUG_AREA_MAIN, "Threads system started");
-	release_pool_threads(); 
+	release_thread_pools();
 	nuauthdatas->is_starting = FALSE;
 }
 
