@@ -2,7 +2,7 @@
  ** Copyright(C) 2005,2006,2007 INL
  ** Written by  Eric Leblond <regit@inl.fr>
  **
- ** $Id: modules.c 3581 2007-08-01 09:45:14Z regit $
+ ** $Id: modules.c 3909 2007-11-07 22:09:19Z regit $
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
@@ -59,6 +59,23 @@
 #include <auth_srv.h>
 #include "modules_definition.h"
 
+/** This is a static variable to initiate all pointers to zero */
+static hook_t hooks[MOD_END] = {
+	{ "nuauth_user_check_module", NULL, NULL, "user_check", "user checking" },
+	{ "nuauth_get_user_id_module", NULL, NULL, "get_user_id", "user id fetching" },
+	{ "nuauth_get_user_groups_module", NULL, NULL, "get_user_groups", "user groups fetching" },
+	{ "nuauth_auth_error_log_module", NULL, NULL, "auth_error_log", "auth error log" },
+	{ "nuauth_acl_check_module", NULL, NULL, "acl_check", "acls checking" },
+	{ "nuauth_user_session_modify_module", NULL, NULL,  "user_session_modify", "user session modify" },
+	{ "nuauth_user_logs_module", NULL, NULL, "user_packet_logs", "user packet logging" },
+	{ "nuauth_user_session_logs_module", NULL, NULL, "user_session_logs", "user session logging" },
+	{ "nuauth_finalize_packet_module", NULL, NULL, "finalize_packet", "finalize packet" },
+	{ "nuauth_periods_module", NULL, NULL, "define_periods", "define periods checking" },
+	{ "nuauth_certificate_check_module", NULL, NULL,  "certificate_check", "certificate check" },
+	{ "nuauth_certificate_to_uid_module", NULL, NULL, "certificate_to_uid", "certificate to uid" },
+	{ "nuauth_ip_authentication_module", NULL, NULL, "ip_authentication", "ip authentication" },
+};
+
 /**
  * Check a user/password against the list of modules used for user authentication
  *  It returns the decision using SASL defined return value.
@@ -67,7 +84,7 @@ int modules_user_check(const char *user, const char *pass,
 		       unsigned passlen)
 {
 	/* iter through module list and stop when user is found */
-	GSList *walker = user_check_modules;
+	GSList *walker = hooks[MOD_USER_CHECK].modules;
 	int walker_return = 0;
 	block_on_conf_reload();
 	for (; walker != NULL; walker = walker->next) {
@@ -90,7 +107,7 @@ int modules_user_check(const char *user, const char *pass,
 GSList *modules_get_user_groups(const char *user)
 {
 	/* iter through module list and stop when an acl is found */
-	GSList *walker = get_user_groups_modules;
+	GSList *walker = hooks[MOD_USER_GROUPS].modules;
 	GSList *walker_return = NULL;
 
 	block_on_conf_reload();
@@ -112,7 +129,7 @@ GSList *modules_get_user_groups(const char *user)
 uint32_t modules_get_user_id(const char *user)
 {
 	/* iter through module list and stop when an acl is found */
-	GSList *walker = get_user_id_modules;
+	GSList *walker = hooks[MOD_USER_ID].modules;
 	uint32_t walker_return = 0;
 
 	block_on_conf_reload();
@@ -138,7 +155,7 @@ uint32_t modules_get_user_id(const char *user)
 GSList *modules_acl_check(connection_t * element)
 {
 	/* iter through module list and stop when an acl is found */
-	GSList *walker = acl_check_modules;
+	GSList *walker = hooks[MOD_ACL_CHECK].modules;
 	GSList *walker_return = NULL;
 
 	block_on_conf_reload();
@@ -160,7 +177,7 @@ GSList *modules_acl_check(connection_t * element)
 gchar *modules_ip_auth(tracking_t * header)
 {
 	/* iter through module list and stop when decision is made */
-	GSList *walker = ip_auth_modules;
+	GSList *walker = hooks[MOD_IP_AUTH].modules;
 	gchar *walker_return = NULL;
 
 	block_on_conf_reload();
@@ -181,7 +198,7 @@ gchar *modules_ip_auth(tracking_t * header)
 int modules_user_logs(void *element, tcp_state_t state)
 {
 	/* iter through all modules list */
-	GSList *walker = user_logs_modules;
+	GSList *walker = hooks[MOD_LOG_PACKETS].modules;
 
 	block_on_conf_reload();
 	for (; walker != NULL; walker = walker->next) {
@@ -200,7 +217,7 @@ int modules_user_logs(void *element, tcp_state_t state)
 int modules_user_session_logs(user_session_t * user, session_state_t state)
 {
 	/* iter through all modules list */
-	GSList *walker = user_session_logs_modules;
+	GSList *walker = hooks[MOD_LOG_SESSION].modules;
 
 	block_on_conf_reload();
 	for (; walker != NULL; walker = walker->next) {
@@ -219,7 +236,7 @@ int modules_user_session_logs(user_session_t * user, session_state_t state)
 void modules_parse_periods(GHashTable * periods)
 {
 	/* iter through all modules list */
-	GSList *walker = period_modules;
+	GSList *walker = hooks[MOD_PERIOD].modules;
 
 	for (; walker != NULL; walker = walker->next) {
 		define_period_callback *handler =
@@ -239,7 +256,7 @@ void modules_parse_periods(GHashTable * periods)
 int modules_check_certificate(gnutls_session session, gnutls_x509_crt cert)
 {
 	/* iter through all modules list */
-	GSList *walker = certificate_check_modules;
+	GSList *walker = hooks[MOD_CERT_CHECK].modules;
 	int ret;
 
 	log_message(VERBOSE_DEBUG, DEBUG_AREA_MAIN, "module check certificate");
@@ -270,7 +287,7 @@ gchar *modules_certificate_to_uid(gnutls_session session,
 				  gnutls_x509_crt cert)
 {
 	/* iter through all modules list */
-	GSList *walker = certificate_to_uid_modules;
+	GSList *walker = hooks[MOD_CERT_TO_UID].modules;
 	gchar *uid;
 
 	block_on_conf_reload();
@@ -295,7 +312,7 @@ gchar *modules_certificate_to_uid(gnutls_session session,
 int modules_user_session_modify(user_session_t * c_session)
 {
 	/* iter through all modules list */
-	GSList *walker = user_session_modify_modules;
+	GSList *walker = hooks[MOD_SESSION_MODIFY].modules;
 
 	block_on_conf_reload();
 	for (; walker != NULL; walker = walker->next) {
@@ -315,7 +332,7 @@ int modules_user_session_modify(user_session_t * c_session)
 nu_error_t modules_finalize_packet(connection_t * connection)
 {
 	/* iter through all modules list */
-	GSList *walker = finalize_packet_modules;
+	GSList *walker = hooks[MOD_FINALIZE_PACKET].modules;
 
 	block_on_conf_reload();
 	for (; walker != NULL; walker = walker->next) {
@@ -334,7 +351,7 @@ nu_error_t modules_finalize_packet(connection_t * connection)
 void modules_auth_error_log(user_session_t * session,
 			    nuauth_auth_error_t error, const char *message)
 {
-	GSList *walker = auth_error_log_modules;
+	GSList *walker = hooks[MOD_USER_FAIL].modules;
 
 	block_on_conf_reload();
 	for (; walker != NULL; walker = walker->next) {
@@ -393,19 +410,6 @@ int init_modules_system()
 {
 	/* init modules list mutex */
 	modules_mutex = g_mutex_new();
-	user_check_modules = NULL;
-	get_user_groups_modules = NULL;
-	get_user_id_modules = NULL;
-	acl_check_modules = NULL;
-	period_modules = NULL;
-	ip_auth_modules = NULL;
-	user_logs_modules = NULL;
-	user_session_logs_modules = NULL;
-	certificate_to_uid_modules = NULL;
-	certificate_check_modules = NULL;
-	user_session_modify_modules = NULL;
-	finalize_packet_modules = NULL;
-	auth_error_log_modules = NULL;
 
 	return 1;
 }
@@ -500,13 +504,13 @@ static int load_modules_from(gchar * confvar, gchar * func,
 			g_error("Unable to load module %s in %s",
 				modules_list[i], MODULE_PATH);
 			free_module_t(current_module);
-			continue;
+			return 0;
 		}
 
 		/* check module version */
 		if (!check_module_version(current_module->module)) {
 			free_module_t(current_module);
-			continue;
+			return 0;
 		}
 
 		/* get module function handler */
@@ -517,7 +521,7 @@ static int load_modules_from(gchar * confvar, gchar * func,
 				g_module_name(current_module->module));
 			free_module_t(current_module);
 			g_strfreev(params_list);
-			continue;
+			return 0;
 		}
 
 		current_module->hook = hook;
@@ -529,14 +533,16 @@ static int load_modules_from(gchar * confvar, gchar * func,
 			/* Initialize module */
 			if (!initmod(current_module)) {
 				g_warning
-				    ("Unable to init module, continuing anyway");
+				    ("Unable to init module");
 				current_module->params = NULL;
+				return 0;
 			}
 		} else {
 			log_message(WARNING, DEBUG_AREA_MAIN,
 				    "No init function for module %s: PLEASE UPGRADE!",
 				    current_module->module_name);
 			current_module->params = NULL;
+			return 0;
 		}
 
 		/* get params for module by calling module exported function */
@@ -547,6 +553,7 @@ static int load_modules_from(gchar * confvar, gchar * func,
 				    "No unload function for module %s: PLEASE UPGRADE!",
 				    current_module->module_name);
 			current_module->free_params = NULL;
+			return 0;
 		}
 
 		/* store module in module list */
@@ -560,7 +567,6 @@ static int load_modules_from(gchar * confvar, gchar * func,
 	}
 	g_strfreev(modules_list);
 	return 1;
-
 }
 
 /**
@@ -592,78 +598,61 @@ int load_modules()
 		{"nuauth_auth_error_log_module", G_TOKEN_STRING, 1,
 		 g_strdup("")}
 	};
-	char *nuauth_acl_check_module;
-	char *nuauth_user_check_module;
-	char *nuauth_get_user_groups_module = NULL;
-	char *nuauth_get_user_id_module = NULL;
-	char *nuauth_user_session_logs_module;
-	char *nuauth_user_logs_module;
-	char *nuauth_ip_authentication_module = NULL;
-	char *nuauth_periods_module;
-	char *nuauth_certificate_check_module;
-	char *nuauth_certificate_to_uid_module;
-	char *nuauth_user_session_modify_module;
-	char *nuauth_finalize_packet_module;
-	char *nuauth_auth_error_log_module;
+	int i;
 	char *configfile = DEFAULT_CONF_FILE;
 
 	/* parse conf file */
-	parse_conffile(configfile,
+	if(!parse_conffile(configfile,
 		       sizeof(nuauth_vars) / sizeof(confparams_t),
-		       nuauth_vars);
+		       nuauth_vars))
+	{
+	        log_message(FATAL, DEBUG_AREA_MAIN, "Failed to load config file %s", configfile);
+		return 0;
+	}
 
 #define READ_CONF(KEY) \
     get_confvar_value(nuauth_vars, sizeof(nuauth_vars)/sizeof(confparams_t), KEY);
 
-	nuauth_user_check_module =
-	    (char *) READ_CONF("nuauth_user_check_module");
-	nuauth_user_session_logs_module =
-	    (char *) READ_CONF("nuauth_user_session_logs_module");
-	nuauth_user_logs_module =
-	    (char *) READ_CONF("nuauth_user_logs_module");
-	nuauth_acl_check_module =
-	    (char *) READ_CONF("nuauth_acl_check_module");
-	nuauth_periods_module =
-	    (char *) READ_CONF("nuauth_periods_module");
+	hooks[MOD_USER_CHECK].config = 
+		(char *) READ_CONF(hooks[MOD_USER_CHECK].configstring);
+	for (i = MOD_SIMPLE; i < MOD_OPTIONNAL; i++) {
+		hooks[i].config = (char *) READ_CONF(hooks[i].configstring);
+	}
+
 	if (nuauthconf->do_ip_authentication) {
-		nuauth_ip_authentication_module =
-		    (char *) READ_CONF("nuauth_ip_authentication_module");
+		hooks[MOD_IP_AUTH].config = 
+			(char *) READ_CONF(hooks[MOD_IP_AUTH].configstring);
 	}
-	nuauth_certificate_check_module =
-	    (char *) READ_CONF("nuauth_certificate_check_module");
-	nuauth_certificate_to_uid_module =
-	    (char *) READ_CONF("nuauth_certificate_to_uid_module");
-	nuauth_user_session_modify_module =
-	    (char *) READ_CONF("nuauth_user_session_modify_module");
-	nuauth_finalize_packet_module =
-	    (char *) READ_CONF("nuauth_finalize_packet_module");
-	nuauth_auth_error_log_module =
-	    (char *) READ_CONF("nuauth_auth_error_log_module");
 
 	/* free config struct */
 	free_confparams(nuauth_vars,
 			sizeof(nuauth_vars) / sizeof(confparams_t));
 
+
 #undef READ_CONF
 #define READ_CONF(KEY) \
     get_confvar_value(deps_check_vars, sizeof(deps_check_vars)/sizeof(confparams_t), KEY);
-
-	if (nuauth_user_check_module) {
+	if (hooks[MOD_USER_CHECK].config) {
 		confparams_t deps_check_vars[] = {
 			{"nuauth_get_user_groups_module", G_TOKEN_STRING, 1,
-				g_strdup(nuauth_user_check_module)},
+				g_strdup(hooks[MOD_USER_CHECK].config)},
 			{"nuauth_get_user_id_module", G_TOKEN_STRING, 1,
-				g_strdup(nuauth_user_check_module)},
+				g_strdup(hooks[MOD_USER_CHECK].config)},
 		};
 
 		/* parse conf file for user_check sub vars*/
-		parse_conffile(configfile,
+		if(!parse_conffile(configfile,
 				sizeof(deps_check_vars) / sizeof(confparams_t),
-				deps_check_vars);
-		nuauth_get_user_groups_module =
-			(char *) READ_CONF("nuauth_get_user_groups_module");
-		nuauth_get_user_id_module =
-			(char *) READ_CONF("nuauth_get_user_id_module");
+				deps_check_vars))
+		{
+		        log_message(FATAL, DEBUG_AREA_MAIN, "Failed to load config file %s", configfile);
+			return 0;
+		}
+
+		hooks[MOD_USER_ID].config = 
+			(char *) READ_CONF(hooks[MOD_USER_ID].configstring);
+		hooks[MOD_USER_GROUPS].config = 
+			(char *) READ_CONF(hooks[MOD_USER_GROUPS].configstring);
 		/* free config struct */
 		free_confparams(deps_check_vars,
 				sizeof(deps_check_vars) / sizeof(confparams_t));
@@ -672,44 +661,22 @@ int load_modules()
 	/* external auth module loading */
 	g_mutex_lock(modules_mutex);
 
-#define LOAD_MODULE(VAR, LIST, KEY, TEXT, HOOK) \
-    log_message(VERBOSE_DEBUG, DEBUG_AREA_MAIN, "Loading " TEXT " modules:"); \
-    load_modules_from(VAR, KEY, &(LIST), HOOK); \
-    g_free(VAR);
+#define LOAD_MODULE(HOOK) \
+    log_message(VERBOSE_DEBUG, DEBUG_AREA_MAIN, "Loading %s modules:", hooks[HOOK].message); \
+    if(!load_modules_from(hooks[HOOK].config, hooks[HOOK].funcstring, &(hooks[HOOK].modules), HOOK)) \
+    { \
+        log_message(FATAL, DEBUG_AREA_MAIN, "Failed to load modules %s", hooks[HOOK].message); \
+    	return 0; \
+    }
+    
 
 	/* loading modules */
-	LOAD_MODULE(nuauth_user_check_module, user_check_modules,
-		    "user_check", "user checking", MOD_USER_CHECK);
-	LOAD_MODULE(nuauth_get_user_groups_module, get_user_groups_modules,
-		    "get_user_groups", "user groups fetching", MOD_USER_GROUPS);
-	LOAD_MODULE(nuauth_get_user_id_module, get_user_id_modules,
-		    "get_user_id", "user id fetching",MOD_USER_ID);
-	LOAD_MODULE(nuauth_acl_check_module, acl_check_modules,
-		    "acl_check", "acls checking", MOD_ACL_CHECK);
-	LOAD_MODULE(nuauth_periods_module, period_modules,
-		    "define_periods", "define periods checking", MOD_PERIOD);
-	LOAD_MODULE(nuauth_user_session_logs_module,
-		    user_session_logs_modules, "user_session_logs",
-		    "user session logging", MOD_LOG_SESSION);
-	LOAD_MODULE(nuauth_user_logs_module, user_logs_modules,
-		    "user_packet_logs", "user packet logging", MOD_LOG_PACKETS);
-	LOAD_MODULE(nuauth_certificate_check_module,
-		    certificate_check_modules, "certificate_check",
-		    "certificate check", MOD_CERT_CHECK);
-	LOAD_MODULE(nuauth_certificate_to_uid_module,
-		    certificate_to_uid_modules, "certificate_to_uid",
-		    "certificate to uid", MOD_CERT_TO_UID);
-	LOAD_MODULE(nuauth_finalize_packet_module, finalize_packet_modules,
-		    "finalize_packet", "finalize packet", MOD_FINALIZE_PACKET);
-	LOAD_MODULE(nuauth_auth_error_log_module, auth_error_log_modules,
-		    "auth_error_log", "auth error log", MOD_USER_FAIL);
-	LOAD_MODULE(nuauth_user_session_modify_module,
-		    user_session_modify_modules, "user_session_modify",
-		    "user session modify", MOD_SESSION_MODIFY);
+	for (i = MOD_FIRST; i < MOD_OPTIONNAL; i++) {
+		LOAD_MODULE(i);
+	}
+
 	if (nuauthconf->do_ip_authentication) {
-		LOAD_MODULE(nuauth_ip_authentication_module,
-			    ip_auth_modules, "ip_authentication",
-			    "ip authentication", MOD_IP_AUTH);
+		LOAD_MODULE(MOD_IP_AUTH);
 	}
 
 	g_mutex_unlock(modules_mutex);
@@ -722,6 +689,7 @@ int load_modules()
 void unload_modules()
 {
 	GSList *c_module;
+	unsigned int i;
 
 	g_mutex_lock(modules_mutex);
 
@@ -740,36 +708,12 @@ void unload_modules()
 	nuauthdatas->modules = NULL;
 
 	/* free all lists */
-	g_slist_free(user_check_modules);
-	user_check_modules = NULL;
-	g_slist_free(get_user_groups_modules);
-	get_user_groups_modules = NULL;
-	g_slist_free(get_user_id_modules);
-	get_user_id_modules = NULL;
-	g_slist_free(acl_check_modules);
-	acl_check_modules = NULL;
-	g_slist_free(period_modules);
-	period_modules = NULL;
-	g_slist_free(ip_auth_modules);
-	ip_auth_modules = NULL;
-	g_slist_free(user_logs_modules);
-	user_logs_modules = NULL;
-	g_slist_free(user_session_logs_modules);
-	user_session_logs_modules = NULL;
+	for(i = 0; i < (sizeof(hooks) / sizeof(*hooks)); ++i) {
+		g_slist_free(hooks[i].modules);
+		hooks[i].modules = NULL;
+		g_free(hooks[i].config);
+	}
 
-	g_slist_free(certificate_check_modules);
-	certificate_check_modules = NULL;
-	g_slist_free(certificate_to_uid_modules);
-	certificate_to_uid_modules = NULL;
-
-	g_slist_free(user_session_modify_modules);
-	user_session_modify_modules = NULL;
-
-	g_slist_free(finalize_packet_modules);
-	finalize_packet_modules = NULL;
-
-	g_slist_free(auth_error_log_modules);
-	auth_error_log_modules = NULL;
 
 	g_mutex_unlock(modules_mutex);
 }
