@@ -4,7 +4,7 @@
  **     written by Eric Leblond <eric@regit.org>
  **                Vincent Deffontaines <vincent@gryzor.com>
  **
- ** $Id: ldap.c 3281 2007-05-09 21:17:46Z regit $
+ ** $Id: ldap.c 4165 2007-12-21 10:33:01Z regit $
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
@@ -204,7 +204,7 @@ G_MODULE_EXPORT gboolean init_module_from_conf(module_t * module)
 
 
 	log_message(VERBOSE_DEBUG, DEBUG_AREA_MAIN,
-		    "Ldap module ($Revision: 3281 $)");
+		    "Ldap module ($Revision: 4165 $)");
 	if (!module->configfile) {
 		configfile = DEFAULT_CONF_FILE;
 	} else {
@@ -321,25 +321,36 @@ static LDAP *ldap_conn_init(struct ldap_params *params)
 	/* init connection */
 	ld = ldap_init(params->ldap_server, params->ldap_server_port);
 	if (!ld) {
-		log_message(WARNING, DEBUG_AREA_MAIN, "ldap init error\n");
+		log_message(WARNING, DEBUG_AREA_MAIN, "Ldap init error");
 		return NULL;
 	}
 	if (ldap_set_option(ld, LDAP_OPT_PROTOCOL_VERSION,
 			    &version) == LDAP_OPT_SUCCESS) {
 		/* Goes to ssl if needed */
+
 #ifdef LDAP_OPT_X_TLS
 		if (params->ldap_server_port == LDAPS_PORT) {
 			int tls_option;
-			tls_option = LDAP_OPT_X_TLS_TRY;
-			ldap_set_option(ld, LDAP_OPT_X_TLS,
+			tls_option = LDAP_OPT_X_TLS_HARD;
+			err = ldap_set_option(ld, LDAP_OPT_X_TLS,
 					(void *) &tls_option);
+			if (err != LDAP_OPT_SUCCESS) {
+				log_message(SERIOUS_WARNING, DEBUG_AREA_AUTH,
+					    "Can not set tls option: %s",
+					    ldap_err2string(err));
+				return NULL;
+			}
 		}
-#endif				/* LDAP_OPT_X_TLS */
+#endif /* LDAP_OPT_X_TLS */
+
 		err =
 		    ldap_bind_s(ld, params->binddn, params->bindpasswd,
 				LDAP_AUTH_SIMPLE);
 		if (err != LDAP_SUCCESS) {
 			if (err == LDAP_SERVER_DOWN) {
+				log_message(INFO, DEBUG_AREA_AUTH,
+					    "Can not connect to ldap: %s",
+					    ldap_err2string(err));
 				/* we lost connection, so disable current one */
 				ldap_unbind(ld);
 				ld = NULL;
@@ -347,7 +358,7 @@ static LDAP *ldap_conn_init(struct ldap_params *params)
 				return NULL;
 			}
 			log_message(SERIOUS_WARNING, DEBUG_AREA_AUTH,
-				    "ldap bind error : %s \n",
+				    "Ldap bind error : %s",
 				    ldap_err2string(err));
 			return NULL;
 		}
@@ -469,7 +480,7 @@ G_MODULE_EXPORT GSList *acl_check(connection_t * element,
 	GSList *temp_list = NULL;
 	char filter[LDAP_QUERY_SIZE];
 	char **attrs_array, **walker;
-	int attrs_array_len, i, group;
+	int attrs_array_len, i, integer;
 	struct timeval timeout;
 	struct acl_group *this_acl;
 	struct weighted_acl *this = NULL;
@@ -734,6 +745,7 @@ G_MODULE_EXPORT GSList *acl_check(connection_t * element,
 				this = g_new0(struct weighted_acl, 1);
 			}
 			g_assert(this_acl);
+			this_acl->users = NULL;
 			this_acl->groups = NULL;
 			this_acl->period = NULL;
 			this_acl->log_prefix = NULL;
@@ -788,19 +800,33 @@ G_MODULE_EXPORT GSList *acl_check(connection_t * element,
 					  this_acl->answer,
 					  this_acl->period);
 			ldap_value_free(attrs_array);
-			/* build groups  list */
+			/* build groups list */
 			attrs_array = ldap_get_values(ld, result, "Group");
 			attrs_array_len = ldap_count_values(attrs_array);
 			walker = attrs_array;
 			for (i = 0; i < attrs_array_len; i++) {
-				sscanf(*walker, "%d", &group);
+				sscanf(*walker, "%d", &integer);
 				this_acl->groups =
 				    g_slist_prepend(this_acl->groups,
 						    GINT_TO_POINTER
-						    (group));
+						    (integer));
 				walker++;
 			}
 			ldap_value_free(attrs_array);
+			/* build users  list */
+			attrs_array = ldap_get_values(ld, result, "User");
+			attrs_array_len = ldap_count_values(attrs_array);
+			walker = attrs_array;
+			for (i = 0; i < attrs_array_len; i++) {
+				sscanf(*walker, "%d", &integer);
+				this_acl->users =
+				    g_slist_prepend(this_acl->users,
+						    GINT_TO_POINTER
+						    (integer));
+				walker++;
+			}
+			ldap_value_free(attrs_array);
+
 			result = ldap_next_entry(ld, result);
 
 			if (nuauthconf->prio_to_nok == 2) {
@@ -808,7 +834,7 @@ G_MODULE_EXPORT GSList *acl_check(connection_t * element,
 			}
 
 			/* add when acl is filled */
-			if (this_acl->groups != NULL) {
+			if (this_acl->groups || this_acl->users) {
 				if (nuauthconf->prio_to_nok == 2) {
 					g_list = g_slist_insert_sorted(g_list,
 							this,
