@@ -4,7 +4,7 @@
  **	       Vincent Deffontaines <vincent@gryzor.com>
  **	       Victor Stinner <haypo@inl.fr>
  **
- ** $Id: mysql.c 3649 2007-08-17 15:46:10Z regit $
+ ** $Id: mysql.c 3999 2007-12-01 22:55:23Z regit $
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
@@ -246,7 +246,7 @@ G_MODULE_EXPORT gboolean init_module_from_conf(module_t * module)
 	    g_new0(struct log_mysql_params, 1);
 
 	log_message(VERBOSE_DEBUG, DEBUG_AREA_MAIN,
-		    "Log_mysql module ($Revision: 3649 $)");
+		    "Log_mysql module ($Revision: 3999 $)");
 	/* init global variables */
 	params->mysql_ssl_cipher = MYSQL_SSL_CIPHER;
 	params->hook = module->hook;
@@ -300,9 +300,9 @@ G_MODULE_EXPORT gboolean init_module_from_conf(module_t * module)
 			sizeof(mysql_nuauth_vars) / sizeof(confparams_t));
 
 	if (params->mysql_admin_bofh) {
-		if (nuauthconf->connect_policy !=  POLICY_ONE_LOGIN) {
+		if (nuauthconf->single_user_client_limit !=  1 ) {
 			log_message(WARNING, DEBUG_AREA_MAIN,
-			    "Resetting mysql_admin_bofh to 0 because multiple login are allowed");
+			    "Resetting mysql_admin_bofh to 0 because multiple logins are allowed");
 			params->mysql_admin_bofh = 0;
 		}
 		/** \todo Reset mysql_admin_bofh if we more than one nufw */
@@ -338,7 +338,7 @@ static MYSQL *mysql_conn_init(struct log_mysql_params *params)
 	/* init connection */
 	ld = mysql_init(ld);
 	if (ld == NULL) {
-		log_message(WARNING, DEBUG_AREA_MAIN, "mysql init error : %s",
+		log_message(WARNING, DEBUG_AREA_MAIN, "mysql init error: %s",
 			    strerror(errno));
 		return NULL;
 	}
@@ -370,7 +370,7 @@ static MYSQL *mysql_conn_init(struct log_mysql_params *params)
 	     params->mysql_passwd, params->mysql_db_name,
 	     params->mysql_server_port, NULL, 0)) {
 		log_message(WARNING, DEBUG_AREA_MAIN,
-			    "mysql connection failed : %s",
+			    "mysql connection failed: %s",
 			    mysql_error(ld));
 		mysql_close(ld);
 		return NULL;
@@ -515,7 +515,7 @@ static char *build_insert_request(MYSQL * ld, connection_t * element,
 				  sizeof(request_fields));
 			ok = secure_snprintf(tmp_buffer,
 					     sizeof(tmp_buffer),
-					     "'%s: %s', '%lu', '%s', '%s', '%s'",
+					     "'%s %s', '%lu', '%s', '%s', '%s'",
 					     log_prefix, auth_oob_prefix,
 					     (long unsigned int) element->
 					     user_id, quoted_username,
@@ -535,7 +535,7 @@ static char *build_insert_request(MYSQL * ld, connection_t * element,
 		g_strlcat(request_fields,
 			  "oob_prefix", sizeof(request_fields));
 		ok = secure_snprintf(tmp_buffer, sizeof(tmp_buffer),
-				     "'%s: %s'",
+				     "'%s %s'",
 				     log_prefix, unauth_oob_prefix);
 		if (!ok) {
 			return NULL;
@@ -691,7 +691,7 @@ static inline int log_state_established(MYSQL * ld,
 		Result = mysql_real_query(ld, request, strlen(request));
 		if (Result != 0) {
 			log_message(SERIOUS_WARNING, DEBUG_AREA_MAIN,
-				    "Can not update Data : %s",
+				    "Can not update Data: %s",
 				    mysql_error(ld));
 			mysql_close_current(params);
 			return -1;
@@ -768,7 +768,7 @@ static inline int log_state_close(MYSQL * ld,
 	Result = mysql_real_query(ld, request, strlen(request));
 	if (Result != 0) {
 		log_message(SERIOUS_WARNING, DEBUG_AREA_MAIN,
-			    "Can not update Data : %s", mysql_error(ld));
+			    "Can not update Data: %s", mysql_error(ld));
 		mysql_close_current(params);
 		return -1;
 	}
@@ -910,16 +910,8 @@ static nu_error_t build_conntrack_msg_from_mysql(MYSQL_ROW row,
 	memset(&(msgdatas->tracking), 0, sizeof(tracking_t));
 	/* fill msgdatas.tracking with datas */
 	if (params->mysql_use_ipv4_schema) {
-		/* convert u32 to IPV6 */
-		msgdatas->tracking.saddr.s6_addr32[0] = 0;
-		msgdatas->tracking.saddr.s6_addr32[1] = 0;
-		msgdatas->tracking.saddr.s6_addr32[2] = 0xffff0000;
-		msgdatas->tracking.saddr.s6_addr32[3] = atol(row[1]);
-
-		msgdatas->tracking.daddr.s6_addr32[0] = 0;
-		msgdatas->tracking.daddr.s6_addr32[1] = 0;
-		msgdatas->tracking.daddr.s6_addr32[2] = 0xffff0000;
-		msgdatas->tracking.daddr.s6_addr32[3] = atol(row[2]);
+		uint32_to_ipv6(atol(row[1]), &msgdatas->tracking.saddr);
+		uint32_to_ipv6(atol(row[2]), &msgdatas->tracking.daddr);
 	} else {
 		memcpy(&(msgdatas->tracking.saddr),
 				row[1],
