@@ -2,7 +2,7 @@
  ** Copyright(C) 2006 INL
  **	written by Victor Stinner <victor.stinner AT haypocalc.com>
  **
- ** $Id: nuprelude.c 3259 2007-05-04 22:34:08Z regit $
+ ** $Id: nuprelude.c 3771 2007-09-25 23:22:48Z regit $
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
@@ -31,6 +31,15 @@
  * @{ */
 
 
+#define NUFW_ANALYZER_MANUFACTURER "http://www.nufw.org/"
+#define NUFW_ANALYZER_CLASS "Firewall"
+#define NUFW_ANALYZER_VERSION NUAUTH_FULL_VERSION
+#define NUFW_ANALYZER_MODEL "NuFW"
+
+#define CLIENT_ANALYZER_NAME "libnuclient"
+#define CLIENT_ANALYZER_MANUFACTURER NUFW_ANALYZER_MANUFACTURER
+#define CLIENT_ANALYZER_CLASS "NuFW client"
+#define CLIENT_ANALYZER_MODEL "NuFW"
 
 GMutex *global_client_mutex;
 prelude_client_t *global_client;	/* private pointer for prelude client connection */
@@ -50,7 +59,7 @@ G_MODULE_EXPORT gchar *unload_module_with_params(gpointer params_ptr)
 }
 
 /**
- * Function called every second to update timer
+ * Function called every second to update timer (Prelude "heartbeat")
  */
 void update_prelude_timer()
 {
@@ -92,15 +101,17 @@ G_MODULE_EXPORT gboolean init_module_from_conf(module_t * module)
 
 
 	log_message(VERBOSE_DEBUG, DEBUG_AREA_MAIN,
-		    "Log_nuprelude module ($Revision: 3259 $)");
+		    "Log_nuprelude module ($Revision: 3771 $)");
 
 	params->packet_tpl = g_private_new((GDestroyNotify) destroy_idmef);
-	params->session_tpl =
-	    g_private_new((GDestroyNotify) destroy_idmef);
+	params->session_tpl = g_private_new((GDestroyNotify) destroy_idmef);
 	module->params = (gpointer) params;
 	return TRUE;
 }
 
+/**
+ * Delete an IDMEF object
+ */
 static void del_idmef_object(idmef_message_t * message, const char *object)
 {
 	idmef_value_t *val;
@@ -115,6 +126,9 @@ static void del_idmef_object(idmef_message_t * message, const char *object)
 	return;
 }
 
+/**
+ * Add an IDMEF object
+ */
 static int add_idmef_object(idmef_message_t * message, const char *object,
 			    const char *value)
 {
@@ -152,23 +166,64 @@ static int add_idmef_object(idmef_message_t * message, const char *object,
 	return ret;
 }
 
+/**
+ * Set default values in an IDMEF template
+ */
 static int feed_template(idmef_message_t * idmef)
 {
-	/* source address/service */
-	add_idmef_object(idmef, "alert.source(0).node.address(0).category",
-			 "ipv6-addr");
-	add_idmef_object(idmef, "alert.source(0).service.ip_version", "6");
-
-	/* target address/service */
-	add_idmef_object(idmef, "alert.target(0).node.address(0).category",
-			 "ipv6-addr");
-	add_idmef_object(idmef, "alert.target(0).service.ip_version", "6");
+	idmef_analyzer_t *client_analyzer, *analyzer;
+	idmef_alert_t *alert;
+	prelude_string_t *string;
+	int ret;
 
 	/* set assessment */
 	add_idmef_object(idmef, "alert.assessment.impact.type", "user");
+
+	/* create analyzer */
+	alert = idmef_message_get_alert(idmef);
+	if (!alert) {
+		return 0;
+	}
+#if 0
+	ret = idmef_alert_new_analyzer(alert, &analyzer, 1);
+	if (ret < 0)
+		return 0;
+#else
+	client_analyzer = prelude_client_get_analyzer(global_client);
+	ret = idmef_analyzer_clone(client_analyzer, &analyzer);
+	if (ret < 0)
+		return 0;
+	idmef_alert_set_analyzer(alert, analyzer, 1); /*IDMEF_LIST_APPEND */
+#endif
+
+	/* configure analyzer */
+	ret = idmef_analyzer_new_model(analyzer, &string);
+	if (ret < 0)
+		return 0;
+	prelude_string_set_constant(string, NUFW_ANALYZER_MODEL);
+
+	ret = idmef_analyzer_new_class(analyzer, &string);
+	if (ret < 0)
+		return 0;
+	prelude_string_set_constant(string, NUFW_ANALYZER_CLASS);
+
+	ret = idmef_analyzer_new_version(analyzer, &string);
+	if (ret < 0)
+		return 0;
+	prelude_string_set_constant(string, NUFW_ANALYZER_VERSION);
+
+	ret = idmef_analyzer_new_manufacturer(analyzer, &string);
+	if (ret < 0)
+		return 0;
+	prelude_string_set_constant(string, NUFW_ANALYZER_MANUFACTURER);
 	return 1;
 }
 
+/**
+ * Create Prelude alert message template.
+ *
+ * \return NULL on error, or new allocated idmef message on succes.
+ */
 static idmef_message_t *create_alert_template()
 {
 	idmef_message_t *idmef;
@@ -181,7 +236,7 @@ static idmef_message_t *create_alert_template()
 	}
 
 	ret = feed_template(idmef);
-	if (ret < 0) {
+	if (!ret) {
 		prelude_perror(ret, "unable to create IDMEF message");
 		idmef_message_destroy(idmef);
 		return NULL;
@@ -189,11 +244,22 @@ static idmef_message_t *create_alert_template()
 	return idmef;
 }
 
+/**
+ * Create Prelude packet message template
+ *
+ * \return NULL on error, or new allocated idmef message on succes.
+ */
 static idmef_message_t *create_packet_template()
 {
-	return create_alert_template();
+	idmef_message_t *idmef = create_alert_template();
+	if (!idmef)
+		return NULL;
+	return idmef;
 }
 
+/**
+ * Set libnuclient as IDMEF source #0: protocol version and service name
+ */
 static void feed_source_libnuclient(idmef_message_t *idmef)
 {
 	add_idmef_object(idmef,
@@ -204,6 +270,9 @@ static void feed_source_libnuclient(idmef_message_t *idmef)
 			 "nufw-client");
 }
 
+/**
+ * Set nuauth as IDMEF target #0: process path and pid, source IPv6, protocol
+ */
 static void feed_target_nuauth(idmef_message_t *idmef)
 {
 	char buffer[50];
@@ -218,21 +287,20 @@ static void feed_target_nuauth(idmef_message_t *idmef)
 
 	secure_snprintf(buffer, sizeof(buffer), "%lu", (unsigned long) getpid());
 	add_idmef_object(idmef, "alert.target(0).process.pid", buffer);
-
 	add_idmef_object(idmef, "alert.target(0).service.port", nuauthconf->userpckt_port);
-
-	/* TODO: Maybe write real IPv6 of nuauth :-) */
-	add_idmef_object(idmef,
-			 "alert.target(0).service.iana_protocol_number",
-			 "6");
-	add_idmef_object(idmef, "alert.target(0).node.address(0).address",
-			 "::1");
 	add_idmef_object(idmef, "alert.target(0).service.protocol", "tcp");
 }
 
+/**
+ * Create Prelude authentication error message template
+ *
+ * \return NULL on error, or new allocated idmef message on succes.
+ */
 static idmef_message_t *create_autherr_template()
 {
 	idmef_message_t *idmef = create_alert_template();
+	if (!idmef)
+		return NULL;
 
 	feed_source_libnuclient(idmef);
 	feed_target_nuauth(idmef);
@@ -240,47 +308,58 @@ static idmef_message_t *create_autherr_template()
 	return idmef;
 }
 
+/**
+ * Create Prelude session message template
+ *
+ * \return NULL on error, or new allocated idmef message on succes.
+ */
 static idmef_message_t *create_session_template()
 {
 	idmef_message_t *idmef = create_alert_template();
+	if (!idmef)
+		return NULL;
 
 	feed_source_libnuclient(idmef);
 	feed_target_nuauth(idmef);
 
-	add_idmef_object(idmef, "alert.additional_data(0).type", "string");
-	add_idmef_object(idmef, "alert.additional_data(0).meaning",
-			 "OS system name");
-	add_idmef_object(idmef, "alert.additional_data(1).type", "string");
-	add_idmef_object(idmef, "alert.additional_data(1).meaning",
-			 "OS release");
-	add_idmef_object(idmef, "alert.additional_data(2).type", "string");
-	add_idmef_object(idmef, "alert.additional_data(2).meaning",
-			 "OS full version");
-
 	return idmef;
 }
 
-int alert_set_time(idmef_message_t *idmef, time_t *creation_timestamp)
+/**
+ * Create an IDMEF message from a template and set common parameters
+ */
+idmef_message_t* create_from_template(idmef_message_t *tpl, connection_t *conn)
 {
+	idmef_message_t *idmef;
 	idmef_alert_t *alert;
 	idmef_time_t *create_time;
 	idmef_time_t *detect_time;
 	time_t now;
 	int ret;
+	time_t *creation_timestamp;
+
+	/* copy the message */
+	if (idmef_message_clone(tpl, &idmef) < 0) {
+		return NULL;
+	}
 
 	now = time(NULL);
 
 	ret = idmef_message_new_alert(idmef, &alert);
 	if (ret < 0) {
+		idmef_message_destroy(idmef);
 		return 0;
 	}
 
 	/* set create time */
-	if (!creation_timestamp) {
+	if (conn) {
+		creation_timestamp = &conn->timestamp;
+	} else {
 		creation_timestamp = &now;
 	}
 	ret = idmef_time_new_from_time(&create_time, creation_timestamp);
 	if (ret < 0) {
+		idmef_message_destroy(idmef);
 		return 0;
 	}
 	idmef_alert_set_create_time(alert, create_time);
@@ -288,12 +367,87 @@ int alert_set_time(idmef_message_t *idmef, time_t *creation_timestamp)
 	/* set detect time */
 	ret = idmef_alert_new_detect_time(alert, &detect_time);
 	if (ret < 0) {
+		idmef_message_destroy(idmef);
 		return 0;
 	}
 	idmef_time_set_from_time(detect_time, &now);
-	return 1;
+	return idmef;
 }
 
+/**
+ * Set operating system informations of a IDMEF message:
+ *  - create an analyzer to store libnuclient informations
+ *  - set name, model, class, manufacturer
+ *  - set OS type and version
+ */
+void set_os_infos(idmef_message_t *idmef, char* osname, char *osrelease, char *osversion)
+{
+	idmef_alert_t *alert;
+	idmef_analyzer_t *analyzer;
+	prelude_string_t *string = NULL;
+	gchar* fullversion;
+	int ret;
+
+	alert = idmef_message_get_alert(idmef);
+	if (!alert)
+		return;
+
+	ret = idmef_alert_new_analyzer(alert, &analyzer, 2);
+	if (ret < 0)
+		return;
+
+
+	/* configure analyzer */
+	ret = idmef_analyzer_new_name(analyzer, &string);
+	if (ret < 0)
+		return;
+	prelude_string_set_constant(string, CLIENT_ANALYZER_NAME);
+
+	ret = idmef_analyzer_new_model(analyzer, &string);
+	if (ret < 0)
+		return;
+	prelude_string_set_constant(string, CLIENT_ANALYZER_MODEL);
+
+	ret = idmef_analyzer_new_class(analyzer, &string);
+	if (ret < 0)
+		return;
+	prelude_string_set_constant(string, CLIENT_ANALYZER_CLASS);
+
+	ret = idmef_analyzer_new_manufacturer(analyzer, &string);
+	if (ret < 0)
+		return;
+	prelude_string_set_constant(string, CLIENT_ANALYZER_MANUFACTURER);
+
+	/* OS informations */
+	ret = idmef_analyzer_new_ostype(analyzer, &string);
+	if (ret < 0)
+		return;
+	prelude_string_set_dup(string, osname);
+
+	fullversion = g_strdup_printf("%s %s", osrelease, osversion);
+	ret = idmef_analyzer_new_osversion(analyzer, &string);
+	if (ret < 0)
+		return;
+	if (fullversion) {
+		prelude_string_set_dup(string, fullversion);
+		g_free(fullversion);
+	} else {
+		prelude_string_set_dup(string, osversion);
+	}
+}
+
+void set_source0_address(idmef_message_t *idmef, struct in6_addr *addr)
+{
+	char ip_ascii[INET6_ADDRSTRLEN];
+	FORMAT_IPV6(addr, ip_ascii);
+	add_idmef_object(idmef,
+			 "alert.source(0).node.address(0).address",
+			 ip_ascii);
+}
+
+/**
+ * Create IDMEF message for NuFW packet message
+ */
 static idmef_message_t *create_message_packet(idmef_message_t * tpl,
 					      tcp_state_t state,
 					      connection_t * conn,
@@ -306,29 +460,25 @@ static idmef_message_t *create_message_packet(idmef_message_t * tpl,
 	char *tmp_buffer;
 	unsigned short psrc, pdst;
 
-	/* duplicate message */
-	idmef = idmef_message_ref(tpl);
-
-	if (!alert_set_time(idmef, &conn->timestamp))
-	{
-		idmef_message_destroy(idmef);
+	idmef = create_from_template(tpl, conn);
+	if (!idmef) {
 		return NULL;
 	}
 
+	if (state == TCP_STATE_DROP)
+		tmp_buffer = "failed";
+	else
+		tmp_buffer = "succeeded";
+	add_idmef_object(idmef, "alert.assessment.impact.completion", tmp_buffer);
 	add_idmef_object(idmef, "alert.classification.text", state_text);
 	add_idmef_object(idmef, "alert.assessment.impact.severity", severity);
 	add_idmef_object(idmef, "alert.assessment.impact.description",
 			 impact);
 
 	/* IP source/dest */
-	inet_ntop (AF_INET6, &conn->tracking.saddr, ip_ascii, sizeof(ip_ascii));
-	add_idmef_object(idmef,
-			 "alert.source(0).node.address(0).address",
-			 ip_ascii);
-	inet_ntop (AF_INET6, &conn->tracking.daddr, ip_ascii, sizeof(ip_ascii));
-	add_idmef_object(idmef,
-			 "alert.target(0).node.address(0).address",
-			 ip_ascii);
+	set_source0_address(idmef, &conn->tracking.saddr);
+	FORMAT_IPV6(&conn->tracking.daddr, ip_ascii);
+	add_idmef_object(idmef, "alert.target(0).node.address(0).address", ip_ascii);
 
 	/* IP protocol */
 	if (secure_snprintf
@@ -376,6 +526,16 @@ static idmef_message_t *create_message_packet(idmef_message_t * tpl,
 		}
 	}
 
+	/* informations about nufw server */
+	if (conn->tls) {
+		add_idmef_object(idmef, "alert.source(1).process.name", "nufw");
+		add_idmef_object(idmef, "alert.source(1).service.protocol", "tcp");
+		add_idmef_object(idmef, "alert.source(1).service.port", nuauthconf->authreq_port);
+		add_idmef_object(idmef, "alert.source(1).service.iana_protocol_number", "6");
+		FORMAT_IPV6(&conn->tls->peername, ip_ascii);
+		add_idmef_object(idmef, "alert.source(1).node.address(0).address", ip_ascii);
+	}
+
 	/* user informations */
 	if (conn->username != NULL) {
 		add_idmef_object(idmef,
@@ -409,35 +569,18 @@ static idmef_message_t *create_message_packet(idmef_message_t * tpl,
 
 	/* os informations */
 	if (conn->os_sysname != NULL) {
-		add_idmef_object(idmef, "alert.additional_data(0).type",
-				 "string");
-		add_idmef_object(idmef, "alert.additional_data(0).meaning",
-				 "OS system name");
-		add_idmef_object(idmef, "alert.additional_data(0).data",
-				 conn->os_sysname);
-		add_idmef_object(idmef, "alert.additional_data(1).type",
-				 "string");
-		add_idmef_object(idmef, "alert.additional_data(1).meaning",
-				 "OS release");
-		add_idmef_object(idmef, "alert.additional_data(1).data",
-				 conn->os_release);
-		add_idmef_object(idmef, "alert.additional_data(2).type",
-				 "string");
-		add_idmef_object(idmef, "alert.additional_data(2).meaning",
-				 "OS full version");
-		add_idmef_object(idmef, "alert.additional_data(2).data",
-				 conn->os_version);
-	} else {
-		del_idmef_object(idmef, "alert.additional_data(0)");
-		del_idmef_object(idmef, "alert.additional_data(1)");
-		del_idmef_object(idmef, "alert.additional_data(2)");
+		set_os_infos(idmef, conn->os_sysname, conn->os_release, conn->os_version);
 	}
 
 	return idmef;
 }
 
+/**
+ * Add NuFW client informations to an IDMEF message: user name and identifier
+ */
 static void add_user_information(idmef_message_t * idmef,
-				 user_session_t * session)
+				 user_session_t * session,
+				 int userid_is_valid)
 {
 	char buffer[50];
 	if (session->user_name != NULL) {
@@ -448,7 +591,7 @@ static void add_user_information(idmef_message_t * idmef,
 		add_idmef_object(idmef,
 				 "alert.source(0).user.user_id(0).name",
 				 session->user_name);
-		if (secure_snprintf
+		if (userid_is_valid && secure_snprintf
 		    (buffer, sizeof(buffer), "%lu", session->user_id)) {
 			add_idmef_object(idmef,
 					 "alert.source(0).user.user_id(0).number",
@@ -459,6 +602,9 @@ static void add_user_information(idmef_message_t * idmef,
 	}
 }
 
+/**
+ * Create IDMEF message for a NuFW session message
+ */
 static idmef_message_t *create_message_session(idmef_message_t * tpl,
 					       user_session_t * session,
 					       char *state_text,
@@ -469,16 +615,13 @@ static idmef_message_t *create_message_session(idmef_message_t * tpl,
 	char buffer[50];
 	char ip_ascii[INET6_ADDRSTRLEN];
 
-	/* duplicate message */
-	idmef = idmef_message_ref(tpl);
-
-	if (!alert_set_time(idmef, NULL))
-	{
-		idmef_message_destroy(idmef);
+	idmef = create_from_template(tpl, NULL);
+	if (!idmef) {
 		return NULL;
 	}
 
 	add_idmef_object(idmef, "alert.classification.text", state_text);
+	add_idmef_object(idmef, "alert.assessment.impact.completion", "succeeded");
 	add_idmef_object(idmef, "alert.assessment.impact.severity", severity);	/* info | low | medium | high */
 	add_idmef_object(idmef, "alert.assessment.impact.description",
 			 impact);
@@ -487,20 +630,17 @@ static idmef_message_t *create_message_session(idmef_message_t * tpl,
 	secure_snprintf(buffer, sizeof(buffer), "%hu", session->sport);
 	add_idmef_object(idmef,	"alert.source(0).service.port", buffer);
 
-	inet_ntop(AF_INET6, &session->addr, ip_ascii, sizeof(ip_ascii));
-	add_idmef_object(idmef,	"alert.source(0).node.address(0).address",
-			ip_ascii);
+	set_source0_address(idmef, &session->addr);
 
 	/* set user informations */
-	add_user_information(idmef, session);
+	add_user_information(idmef, session, 1);
+
+	FORMAT_IPV6(&session->server_addr, ip_ascii);
+	add_idmef_object(idmef,
+			"alert.target(0).node.address(0).address", ip_ascii);
 
 	/* os informations */
-	add_idmef_object(idmef, "alert.additional_data(0).data",
-			 session->sysname);
-	add_idmef_object(idmef, "alert.additional_data(1).data",
-			 session->release);
-	add_idmef_object(idmef, "alert.additional_data(2).data",
-			 session->version);
+	set_os_infos(idmef, session->sysname, session->release, session->version);
 	return idmef;
 }
 
@@ -510,36 +650,35 @@ static idmef_message_t *create_message_autherr(idmef_message_t * tpl,
 					       const char *severity)
 {
 	idmef_message_t *idmef;
-	char ip_ascii[INET6_ADDRSTRLEN];
 	char buffer[50];
+	char ip_ascii[INET6_ADDRSTRLEN];
 
-	/* duplicate message */
-	idmef = idmef_message_ref(tpl);
-
-	if (!alert_set_time(idmef, NULL))
-	{
-		idmef_message_destroy(idmef);
+	idmef = create_from_template(tpl, NULL);
+	if (!idmef) {
 		return NULL;
 	}
 
+	add_idmef_object(idmef, "alert.assessment.impact.completion", "failed");
 	add_idmef_object(idmef, "alert.assessment.impact.severity",
 			 severity);
 
 	add_idmef_object(idmef, "alert.classification.text",
-			 "Authentification error");
+			 "Authentication error");
 	add_idmef_object(idmef, "alert.assessment.impact.description",
 			 text);
 
 	/* source address */
-	inet_ntop(AF_INET6, &session->addr, ip_ascii, sizeof(ip_ascii));
-	add_idmef_object(idmef, "alert.source(0).node.address(0).address",
-			ip_ascii);
+	set_source0_address(idmef, &session->addr);
 
 	secure_snprintf(buffer, sizeof(buffer), "%hu", session->sport);
 	add_idmef_object(idmef,	"alert.source(0).service.port", buffer);
 
+	FORMAT_IPV6(&session->server_addr, ip_ascii);
+	add_idmef_object(idmef,
+			"alert.target(0).node.address(0).address", ip_ascii);
+
 	/* set user informations */
-	add_user_information(idmef, session);
+	add_user_information(idmef, session, 0);
 
 	return idmef;
 }
@@ -582,13 +721,14 @@ G_MODULE_EXPORT gint user_packet_logs(connection_t * element,
 		break;
 	default:
 		return -1;
-		break;
 	}
 
 	/* get message template (or create it if needed) */
 	tpl = g_private_get(params->packet_tpl);
-	if (tpl == NULL) {
+	if (!tpl) {
 		tpl = create_packet_template();
+		if (!tpl)
+			return -1;
 		g_private_set(params->packet_tpl, tpl);
 	}
 
@@ -596,7 +736,7 @@ G_MODULE_EXPORT gint user_packet_logs(connection_t * element,
 	message =
 	    create_message_packet(tpl, state, element, state_text, impact,
 				  severity);
-	if (message == NULL) {
+	if (!message) {
 		return -1;
 	}
 
@@ -615,19 +755,19 @@ G_MODULE_EXPORT int user_session_logs(user_session_t * c_session,
 	struct log_prelude_params *params = params_ptr;
 	idmef_message_t *tpl;
 	idmef_message_t *message;
-	char *impact;
+	char *impact = NULL;
 	char *severity;
 	char *state_text;
 
 	severity = "low";
 	switch (state) {
 	case SESSION_OPEN:
-		state_text = "user log in";
-		impact = "user log in";
+		state_text = "User log in";
+		impact = g_strdup_printf("User \"%s\" log in", c_session->user_name);
 		break;
 	case SESSION_CLOSE:
-		state_text = "user log out";
-		impact = "user log out";
+		state_text = "User log out";
+		impact = g_strdup_printf("User \"%s\" log out", c_session->user_name);
 		break;
 	default:
 		return -1;
@@ -635,8 +775,12 @@ G_MODULE_EXPORT int user_session_logs(user_session_t * c_session,
 
 	/* get message template (or create it if needed) */
 	tpl = g_private_get(params->session_tpl);
-	if (tpl == NULL) {
+	if (!tpl) {
 		tpl = create_session_template();
+		if (!tpl) {
+			g_free(impact);
+			return -1;
+		}
 		g_private_set(params->session_tpl, tpl);
 	}
 
@@ -644,7 +788,8 @@ G_MODULE_EXPORT int user_session_logs(user_session_t * c_session,
 	message =
 	    create_message_session(tpl, c_session, state_text, impact,
 				   severity);
-	if (message == NULL) {
+	g_free(impact);
+	if (!message) {
 		return -1;
 	}
 
@@ -664,9 +809,11 @@ G_MODULE_EXPORT int user_session_logs(user_session_t * c_session,
 G_MODULE_EXPORT gchar *g_module_check_init()
 {
 	const char *version;
-	int argc;
-	char **argv = NULL;
+	int argc = 1;
+	char *argv[2];
 	int ret;
+	argv[0] = nuauthdatas->program_fullpath;
+	argv[1] = NULL;
 
 	log_message(SERIOUS_WARNING, DEBUG_AREA_MAIN,
 		    "[+] Prelude log: Init Prelude library");
@@ -714,19 +861,6 @@ G_MODULE_EXPORT gchar *g_module_check_init()
 	}
 
 	cleanup_func_push(update_prelude_timer);
-
-#if 0
-	/* set flags */
-	ret = prelude_client_set_flags(global_client,
-				       PRELUDE_CLIENT_FLAGS_ASYNC_SEND |
-				       PRELUDE_CLIENT_FLAGS_ASYNC_TIMER);
-	if (ret < 0) {
-		log_message(WARNING, DEBUG_AREA_MAIN,
-			    "Prelude: Warning, unnable to set asynchronous send and timer: %s",
-			    prelude_strerror(ret));
-	}
-#endif
-
 	return NULL;
 }
 
@@ -741,8 +875,10 @@ G_MODULE_EXPORT void auth_error_log(user_session_t * session,
 
 	/* get message template (or create it if needed) */
 	tpl = g_private_get(params->autherr_tpl);
-	if (tpl == NULL) {
+	if (!tpl) {
 		tpl = create_autherr_template();
+		if (!tpl)
+			return;
 		g_private_set(params->autherr_tpl, tpl);
 	}
 
@@ -752,7 +888,7 @@ G_MODULE_EXPORT void auth_error_log(user_session_t * session,
 	else
 		severity = "medium";
 	message = create_message_autherr(tpl, session, text, severity);
-	if (message == NULL) {
+	if (!message) {
 		return;
 	}
 
