@@ -2,7 +2,7 @@
  **  "plaintext" module
  ** Copyright(C) 2004-2005 Mikael Berthe <mikael+nufw@lists.lilotux.net>
  **
- ** $Id: plaintext.c 3629 2007-08-14 13:31:23Z regit $
+ ** $Id: plaintext.c 4241 2008-01-07 09:20:15Z haypo $
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
@@ -92,7 +92,7 @@ static int parse_ints(char *intline, GSList ** p_intlist, char *prefix)
 			/*  We can't read a number.  This will be an error only if we can */
 			/*   see a comma next. */
 			if (p_nextint) {
-				log_message(WARNING, DEBUG_AREA_MAIN,
+				log_message(FATAL, DEBUG_AREA_MAIN,
 					    "%s parse_ints: Malformed line",
 					    prefix);
 				*p_intlist = intlist;
@@ -144,7 +144,7 @@ static int parse_ports(char *portsline, GSList ** p_portslist,
 			/*  We can't read a port number.  This will be an error only if we can */
 			/*   see a comma next. */
 			if (p_nextports) {
-				log_message(WARNING, DEBUG_AREA_MAIN,
+				log_message(FATAL, DEBUG_AREA_MAIN,
 					    "%s parse_ports: Malformed line",
 					    prefix);
 				*p_portslist = portslist;
@@ -188,22 +188,6 @@ static int parse_ports(char *portsline, GSList ** p_portslist,
 	return 0;
 }
 
-/**
- * Compare addr1 with (addr2 & netmask)
- *
- * \return 0 if they match, integer different than zero otherwise (memcmp result)
- */
-static int compare_ipv6_with_mask(struct in6_addr *addr1, struct in6_addr *addr2,
-			   struct in6_addr *mask)
-{
-	struct in6_addr masked = *addr2;
-	masked.s6_addr32[0] &= mask->s6_addr32[0];
-	masked.s6_addr32[1] &= mask->s6_addr32[1];
-	masked.s6_addr32[2] &= mask->s6_addr32[2];
-	masked.s6_addr32[3] &= mask->s6_addr32[3];
-	return memcmp(addr1, &masked, sizeof(masked));
-}
-
 /**
  * Try to match an address from an IP/mask list.
  *
@@ -213,6 +197,8 @@ static int compare_ipv6_with_mask(struct in6_addr *addr1, struct in6_addr *addr2
  */
 static int match_ip(GSList * ip_list, struct in6_addr *addr)
 {
+	if (!ip_list)
+		return 1;
 	for (; ip_list != NULL; ip_list = g_slist_next(ip_list)) {
 		struct plaintext_ip *item = (struct plaintext_ip *) ip_list->data;
 		if (compare_ipv6_with_mask
@@ -230,19 +216,20 @@ static int match_ip(GSList * ip_list, struct in6_addr *addr)
  */
 static int parse_ips(char *ipsline, GSList ** ip_list, char *prefix)
 {
-	char *p_nextip = NULL;
+	char addr_ascii[INET6_ADDRSTRLEN];
+	char mask_ascii[INET6_ADDRSTRLEN];
 	struct in_addr ip_addr4;
-	uint32_t *p_netmask;
 	struct in6_addr ip_addr6;
 	char *p_tmp;
 	gchar **ip_items = g_strsplit(ipsline, ",", 0);
 	gchar **iter = ip_items;
 	gchar *line;
 	struct plaintext_ip this_ip, *this_ip_copy;
+	int result = 0;
 
 	/*  parsing IPs */
 	for (iter = ip_items; iter != NULL && *iter != NULL; iter++) {
-		uint32_t mask = 0;
+		int32_t mask = 0;
 		int n;
 
 		line = strip_line(*iter, FALSE);
@@ -253,85 +240,60 @@ static int parse_ips(char *ipsline, GSList ** ip_list, char *prefix)
 			*p_tmp++ = 0;
 			n = sscanf(p_tmp, "%u", &mask);
 			if (n != 1) {
-				log_message(WARNING, DEBUG_AREA_MAIN,
+				log_message(FATAL, DEBUG_AREA_MAIN,
 					    "plaintext warning: wrong network mask (%s)",
 					    p_tmp);
-				continue;
+				result = 1;
+				break;
 			}
 		} else {	/*  no -> default netmask is 32 bits */
 			mask = 128;
 		}
 
 		if (0 < inet_pton(AF_INET, line, &ip_addr4)) {
-			this_ip.addr.s6_addr32[0] = 0;
-			this_ip.addr.s6_addr32[1] = 0;
-			this_ip.addr.s6_addr32[2] = 0xffff0000;
-			this_ip.addr.s6_addr32[3] = ip_addr4.s_addr;
-			if (32 < mask)
-				mask = 32;
+			ipv4_to_ipv6(ip_addr4, &this_ip.addr);
 			mask += (128 - 32);
 		} else if (0 < inet_pton(AF_INET6, line, &ip_addr6)) {
 			this_ip.addr = ip_addr6;
 		} else {
 			/*  We can't read an IP address.  This will be an error only if we can */
 			/*   see a comma next. */
-			if (p_nextip) {
-				log_message(WARNING, DEBUG_AREA_MAIN,
-					    "%s parse_ips: Malformed line",
-					    prefix);
-			}
-			log_message(WARNING, DEBUG_AREA_MAIN,
-				    "%s parse_ips: Garbarge at end of line",
-				    prefix);
-			continue;
+			log_message(FATAL, DEBUG_AREA_MAIN,
+				    "%s parse_ips: Unable to parse IP address: %s",
+				    prefix, line);
+			result = 1;
+			break;
 		}
 
-		if (128 < mask)
-			mask = 128;
-
 		/*  Create netmask IPv6 address from netmask in bits */
-		memset(&this_ip.netmask, 0, sizeof(this_ip.netmask));
-		p_netmask = &this_ip.netmask.s6_addr32[0];
-		for (; 32 < mask; mask -= 32) {
-			*p_netmask = 0xffffffff;
-			p_netmask++;
-		}
-		if (mask != 0) {
-			*p_netmask = htonl(0xFFFFFFFF << (32 - mask));
-		}
+		create_ipv6_netmask(&this_ip.netmask, mask);
 
 		if (compare_ipv6_with_mask
 		    (&this_ip.addr, &this_ip.addr,
 		     &this_ip.netmask) != 0) {
-			log_message(WARNING, DEBUG_AREA_MAIN,
-				    "%s parse_ips: Invalid network specification!",
-				    prefix);
-			continue;
+			FORMAT_IPV6(&this_ip.addr, addr_ascii);
+			FORMAT_IPV6(&this_ip.netmask, mask_ascii);
+			log_message(FATAL, DEBUG_AREA_MAIN,
+				    "%s parse_ips: Invalid network specification: IP=%s, netmask=%s!",
+				    prefix, addr_ascii, mask_ascii);
+			result = 1;
+			break;
 		}
 
 		this_ip_copy = g_memdup(&this_ip, sizeof(this_ip));
 		*ip_list = g_slist_prepend(*ip_list, this_ip_copy);
 
 #ifdef DEBUG_ENABLE
-		{
-			char addr_ascii[INET6_ADDRSTRLEN];
-			char mask_ascii[INET6_ADDRSTRLEN];
-			if (inet_ntop
-			    (PF_INET6, &this_ip_copy->addr, addr_ascii,
-			     sizeof(addr_ascii)) != NULL
-			    && inet_ntop(PF_INET6, &this_ip_copy->netmask,
-					 mask_ascii,
-					 sizeof(mask_ascii)) != NULL) {
-				log_message(VERBOSE_DEBUG, DEBUG_AREA_MAIN,
-					    "%s Adding IP = %s, netmask = %s",
-					    prefix, addr_ascii,
-					    mask_ascii);
-			}
-		}
+		FORMAT_IPV6(&this_ip_copy->addr, addr_ascii);
+		FORMAT_IPV6(&this_ip_copy->netmask, mask_ascii);
+		log_message(VERBOSE_DEBUG, DEBUG_AREA_MAIN,
+				"%s Adding IP = %s, netmask = %s",
+				prefix, addr_ascii,
+				mask_ascii);
 #endif
 	}
 	g_strfreev(ip_items);
-	return 0;
+ 	return result;
 }
 
 /**
@@ -479,7 +441,7 @@ static int read_acl_list(struct plaintext_params *params)
 	fd = fopen(params->plaintext_aclfile, "r");
 
 	if (!fd) {
-		log_message(WARNING, DEBUG_AREA_MAIN,
+		log_message(FATAL, DEBUG_AREA_MAIN,
 			    "read_acl_list: fopen error");
 		return 1;
 	}
@@ -499,10 +461,12 @@ static int read_acl_list(struct plaintext_params *params)
 						  newacl->aclname);
 				/*  check if ACL node has minimal information */
 				/*  Warning: this code is duplicated after the loop */
-				if (!newacl->groups) {
+				if (!(newacl->groups || newacl->users)) {
 					log_message(WARNING, DEBUG_AREA_MAIN,
-						    "No group(s) declared in ACL %s",
-						    newacl->aclname);
+						    "L.%d: No user or group(s) declared in ACL %s",
+						    ln, newacl->aclname);
+					fclose(fd);
+					return 2;
 				} else if (newacl->proto == IPPROTO_TCP ||
 					   newacl->proto == IPPROTO_UDP ||
 					   newacl->proto == IPPROTO_ICMP) {
@@ -523,7 +487,7 @@ static int read_acl_list(struct plaintext_params *params)
 
 			p_tmp = strchr(++p_key, ']');
 			if (!p_tmp) {
-				log_message(WARNING, DEBUG_AREA_MAIN,
+				log_message(FATAL, DEBUG_AREA_MAIN,
 					    "L.%d: Malformed line (ACLname)",
 					    ln);
 				fclose(fd);
@@ -533,16 +497,18 @@ static int read_acl_list(struct plaintext_params *params)
 			/*  Ok, new ACL declaration here.  Let's allocate a structure! */
 			newacl = g_new0(struct plaintext_acl, 1);
 			if (!newacl) {
-				log_message(WARNING, DEBUG_AREA_MAIN,
+				log_message(FATAL, DEBUG_AREA_MAIN,
 					    "read_acl_list: Cannot allocate plaintext_acl!");
 				fclose(fd);
 				return 5;
 			}
 
 			newacl->aclname = g_strdup(p_key);
+			newacl->proto = IPPROTO_TCP;
 			newacl->period = NULL;
 			newacl->log_prefix = NULL;
 			newacl->flags = ACL_FLAGS_NONE;
+			newacl->decision = DECISION_ACCEPT;
 			debug_log_message(VERBOSE_DEBUG, DEBUG_AREA_MAIN,
 					  "L.%d: ACL name found: [%s]", ln,
 					  newacl->aclname);
@@ -552,7 +518,7 @@ static int read_acl_list(struct plaintext_params *params)
 
 		/*  We shouldn't be here if we aren't in an ACL declaration */
 		if (!newacl) {
-			log_message(WARNING, DEBUG_AREA_MAIN,
+			log_message(FATAL, DEBUG_AREA_MAIN,
 				    "L.%d: Malformed line (Not in an ACL declaration)",
 				    ln);
 			fclose(fd);
@@ -561,7 +527,7 @@ static int read_acl_list(struct plaintext_params *params)
 
 		p_value = strchr(p_key, '=');
 		if (!p_value) {
-			log_message(WARNING, DEBUG_AREA_MAIN,
+			log_message(FATAL, DEBUG_AREA_MAIN,
 				    "L.%d: Malformed line (No '=' inside)",
 				    ln);
 			fclose(fd);
@@ -588,7 +554,7 @@ static int read_acl_list(struct plaintext_params *params)
 				break;
 			default:
 				{
-					log_message(WARNING, DEBUG_AREA_MAIN,
+					log_message(FATAL, DEBUG_AREA_MAIN,
 						    "L.%d: Malformed line (decision should be 0 or 1)",
 						    ln);
 					fclose(fd);
@@ -608,14 +574,33 @@ static int read_acl_list(struct plaintext_params *params)
 				fclose(fd);
 				return 2;
 			}
+		} else if (!strcasecmp("uid", p_key)) {	/*  Users */
+			char log_prefix[16];
+			snprintf(log_prefix, sizeof(log_prefix) - 1,
+				 "L.%d: ", ln);
+			/*  parsing groups */
+			if (parse_ints
+			    (p_value, &newacl->users, log_prefix)) {
+				fclose(fd);
+				return 2;
+			}
 		} else if (!strcasecmp("proto", p_key)) {	/*  Protocol */
 			if (sscanf(p_value, "%d", &newacl->proto) != 1) {
-				log_message(WARNING, DEBUG_AREA_MAIN,
+				log_message(FATAL, DEBUG_AREA_MAIN,
 					    "L.%d: Malformed line (proto should be a number)",
 					    ln);
 				fclose(fd);
 				return 2;
 			}
+			if (newacl->proto != IPPROTO_TCP && newacl->proto != IPPROTO_UDP
+					&& newacl->proto != IPPROTO_ICMP) {
+				log_message(FATAL, DEBUG_AREA_MAIN,
+					    "L.%d: Unsupported protocol: %d",
+					    ln, newacl->proto);
+				fclose(fd);
+				return 2;
+			}
+
 			debug_log_message(VERBOSE_DEBUG, DEBUG_AREA_MAIN,
 					  "L.%d: Read proto = %d", ln,
 					  newacl->proto);
@@ -741,7 +726,7 @@ static int read_acl_list(struct plaintext_params *params)
 					  newacl->log_prefix);
 		} else if (!strcasecmp("flags", p_key)) {
 			if (sscanf(p_value, "%d", &newacl->flags) != 1) {
-				log_message(WARNING, DEBUG_AREA_MAIN,
+				log_message(FATAL, DEBUG_AREA_MAIN,
 						"L.%d: Malformed line (flags should be a number)",
 						ln);
 				fclose(fd);
@@ -761,9 +746,9 @@ static int read_acl_list(struct plaintext_params *params)
 				  "Done with ACL [%s]", newacl->aclname);
 		/*  check if ACL node has minimal information */
 		/*  Warning: this code is duplicated after the loop */
-		if (!newacl->groups) {
+		if (!(newacl->groups || newacl->users)) {
 			log_message(WARNING, DEBUG_AREA_MAIN,
-				    "No group(s) declared in ACL %s",
+				    "No user or group(s) declared in ACL %s",
 				    newacl->aclname);
 		} else if (newacl->proto == IPPROTO_TCP
 			   || newacl->proto == IPPROTO_UDP
@@ -864,6 +849,7 @@ G_MODULE_EXPORT gboolean unload_module_with_params(struct plaintext_params
 			g_slist_foreach(p_acl->dst_ports, (GFunc) g_free,
 					NULL);
 			g_slist_free(p_acl->dst_ports);
+			g_slist_free(p_acl->users);
 			g_slist_free(p_acl->groups);
 			g_free(p_acl->aclname);
 			g_free(p_acl->period);
@@ -892,7 +878,7 @@ G_MODULE_EXPORT gboolean init_module_from_conf(module_t * module)
 
 
 	log_message(VERBOSE_DEBUG, DEBUG_AREA_MAIN,
-		    "Plaintext module ($Revision: 3629 $)");
+		    "Plaintext module ($Revision: 4241 $)");
 	/*  parse conf file */
 	if (module->configfile) {
 		parse_conffile(module->configfile,
@@ -925,6 +911,31 @@ G_MODULE_EXPORT gboolean init_module_from_conf(module_t * module)
 			sizeof(confparams_t));
 
 	module->params = (gpointer) params;
+
+	/*  Depending on the use of the module load user list or acl list */
+	if (module->hook == MOD_USER_CHECK || module->hook == MOD_USER_ID || module->hook == MOD_USER_GROUPS) {
+		/*  Initialization of the user list */
+		if( read_user_list(params)) {
+			log_message(FATAL, DEBUG_AREA_AUTH,
+				    "Can't parse users file [%s]",
+				    ((struct plaintext_params *) params)->
+				    plaintext_userfile);
+			return FALSE;
+		}
+	} else if (module->hook == MOD_ACL_CHECK) {
+		/*  Initialization of the ACL list */
+		if (read_acl_list(params)) {
+			log_message(SERIOUS_WARNING, DEBUG_AREA_MAIN,
+				    "Can't parse ACLs file [%s]",
+				    ((struct plaintext_params *) params)->
+				    plaintext_aclfile);
+			return FALSE;
+		}
+	} else  {
+		log_message(CRITICAL, DEBUG_AREA_MAIN, "Wrong plugin use: %i", module->hook);
+		return FALSE;
+	}
+
 	return TRUE;
 }
 
@@ -972,23 +983,6 @@ G_MODULE_EXPORT int user_check(const char *username,
 {
 	GSList *res;
 	char *realpass;
-	int initstatus;
-	static GStaticMutex plaintext_initmutex = G_STATIC_MUTEX_INIT;
-
-	/* init has only to be done once */
-	g_static_mutex_lock(&plaintext_initmutex);
-	/*  Initialization if the user list is empty */
-	if (!((struct plaintext_params *) params)->plaintext_userlist) {
-		initstatus = read_user_list(params);
-		if (initstatus) {
-			log_message(SERIOUS_WARNING, DEBUG_AREA_AUTH,
-				    "Can't parse users file [%s]",
-				    ((struct plaintext_params *) params)->
-				    plaintext_userfile);
-			return SASL_BADAUTH;
-		}
-	}
-	g_static_mutex_unlock(&plaintext_initmutex);
 
 	res = fill_user_by_username(username, params);
 	if (res == NULL) {
@@ -1025,27 +1019,10 @@ G_MODULE_EXPORT int user_check(const char *username,
 G_MODULE_EXPORT uint32_t get_user_id(const char *username, gpointer params)
 {
 	GSList *res;
-	int initstatus;
-	static GStaticMutex plaintext_initmutex = G_STATIC_MUTEX_INIT;
-	/* init has only to be done once */
-	g_static_mutex_lock(&plaintext_initmutex);
-	/*  Initialization if the user list is empty */
-	if (!((struct plaintext_params *) params)->plaintext_userlist) {
-		initstatus = read_user_list(params);
-		if (initstatus) {
-			log_message(SERIOUS_WARNING, DEBUG_AREA_AUTH,
-				    "Can't parse users file [%s]",
-				    ((struct plaintext_params *) params)->
-				    plaintext_userfile);
-			return SASL_BADAUTH;
-		}
-	}
-	g_static_mutex_unlock(&plaintext_initmutex);
 
 	res = fill_user_by_username(username, params);
-	if (res == NULL) {
+	if (res == NULL)
 		return 0;
-	}
 
 	return ((struct plaintext_user *) res->data)->uid;
 }
@@ -1055,22 +1032,6 @@ G_MODULE_EXPORT GSList *get_user_groups(const char *username,
 					gpointer params)
 {
 	GSList *res;
-	int initstatus;
-	static GStaticMutex plaintext_initmutex = G_STATIC_MUTEX_INIT;
-	/* init has only to be done once */
-	g_static_mutex_lock(&plaintext_initmutex);
-	/*  Initialization if the user list is empty */
-	if (!((struct plaintext_params *) params)->plaintext_userlist) {
-		initstatus = read_user_list(params);
-		if (initstatus) {
-			log_message(SERIOUS_WARNING, DEBUG_AREA_AUTH,
-				    "Can't parse users file [%s]",
-				    ((struct plaintext_params *) params)->
-				    plaintext_userfile);
-			return NULL;
-		}
-	}
-	g_static_mutex_unlock(&plaintext_initmutex);
 
 	res = fill_user_by_username(username, params);
 	if (res == NULL) {
@@ -1088,24 +1049,6 @@ G_MODULE_EXPORT GSList *acl_check(connection_t * element, gpointer params)
 	struct acl_group *this_acl;
 	tracking_t *netdata = &element->tracking;
 	struct plaintext_acl *p_acl;
-	int initstatus;
-	static GStaticMutex plaintext_initmutex = G_STATIC_MUTEX_INIT;
-
-	/* init has only to be done once */
-	g_static_mutex_lock(&plaintext_initmutex);
-	/*  Initialization if the ACL list is empty */
-	if (!((struct plaintext_params *) params)->plaintext_acllist) {
-		initstatus =
-		    read_acl_list((struct plaintext_params *) params);
-		if (initstatus) {
-			log_message(SERIOUS_WARNING, DEBUG_AREA_MAIN,
-				    "Can't parse ACLs file [%s]",
-				    ((struct plaintext_params *) params)->
-				    plaintext_aclfile);
-			return NULL;
-		}
-	}
-	g_static_mutex_unlock(&plaintext_initmutex);
 
 	/*  netdata.protocol     IPPROTO_TCP || IPPROTO_UDP || IPPROTO_ICMP */
 	/*  netdata.type         for ICMP */
@@ -1120,16 +1063,26 @@ G_MODULE_EXPORT GSList *acl_check(connection_t * element, gpointer params)
 	     p_acllist; p_acllist = g_slist_next(p_acllist)) {
 		p_acl = (struct plaintext_acl *) p_acllist->data;
 
-		if (netdata->protocol != p_acl->proto)
+		debug_log_message(VERBOSE_DEBUG, DEBUG_AREA_MAIN,
+				"(DBG) [plaintext] test acl %s", p_acl->aclname);
+
+		if (netdata->protocol != p_acl->proto) {
+			debug_log_message(VERBOSE_DEBUG, DEBUG_AREA_MAIN,
+					"(DBG) skip ACL %s: protocol doesn't match", p_acl->aclname);
 			continue;
+		}
 
 		/*  Check source address */
 		if (!match_ip(p_acl->src_ip, &netdata->saddr)) {
+			debug_log_message(VERBOSE_DEBUG, DEBUG_AREA_MAIN,
+					"(DBG) skip ACL %s: source IP doesn't match", p_acl->aclname);
 			continue;
 		}
 
 		/*  Check destination address */
 		if (!match_ip(p_acl->dst_ip, &netdata->daddr)) {
+			debug_log_message(VERBOSE_DEBUG, DEBUG_AREA_MAIN,
+					"(DBG) skip ACL %s: destination IP doesn't match", p_acl->aclname);
 			continue;
 		}
 
@@ -1146,19 +1099,13 @@ G_MODULE_EXPORT GSList *acl_check(connection_t * element, gpointer params)
 						break;
 					}
 				}
-				if (!found)
+				if (!found) {
+					debug_log_message(VERBOSE_DEBUG, DEBUG_AREA_MAIN,
+							"(DBG) skip ACL %s: ICMP type doesn't match", p_acl->aclname);
 					continue;
+				}
 			}
 		} else {
-			/*  Following is only for TCP / UDP  (ports stuff...) */
-			if (p_acl->proto != IPPROTO_TCP
-			    && p_acl->proto != IPPROTO_UDP) {
-				g_message
-				    ("[plaintext] Unsupported protocol: %d",
-				     p_acl->proto);
-				continue;
-			}
-
 			/*  Check source port */
 			if (p_acl->src_ports) {
 				int found = 0;
@@ -1180,8 +1127,11 @@ G_MODULE_EXPORT GSList *acl_check(connection_t * element, gpointer params)
 						break;
 					}
 				}
-				if (!found)
+				if (!found) {
+					debug_log_message(VERBOSE_DEBUG, DEBUG_AREA_MAIN,
+							"(DBG) skip ACL %s: TCP/UDP source port doesn't match", p_acl->aclname);
 					continue;
+				}
 			}
 			/*  Check destination port */
 			if (p_acl->dst_ports) {
@@ -1205,6 +1155,8 @@ G_MODULE_EXPORT GSList *acl_check(connection_t * element, gpointer params)
 					}
 				}
 				if (!found) {
+					debug_log_message(VERBOSE_DEBUG, DEBUG_AREA_MAIN,
+							"(DBG) skip ACL %s: TCP/UDP destination port doesn't match", p_acl->aclname);
 					continue;
 				}
 			}
@@ -1262,8 +1214,11 @@ G_MODULE_EXPORT GSList *acl_check(connection_t * element, gpointer params)
 			debug_log_message(VERBOSE_DEBUG, DEBUG_AREA_MAIN,
 					  "(DBG) Checking OS sysname ACL found=%d",
 					  found);
-			if (!found)
+			if (!found) {
+				debug_log_message(VERBOSE_DEBUG, DEBUG_AREA_MAIN,
+						"(DBG) skip ACL %s: OS doesn't match", p_acl->aclname);
 				continue;
+			}
 			log_message(VERBOSE_DEBUG, DEBUG_AREA_MAIN,
 				    "[plaintext] OS match (%s)",
 				    element->os_sysname);
@@ -1293,8 +1248,11 @@ G_MODULE_EXPORT GSList *acl_check(connection_t * element, gpointer params)
 			log_message(VERBOSE_DEBUG, DEBUG_AREA_MAIN,
 				    "(DBG) Checking App ACL found=%d",
 				    found);
-			if (!found)
+			if (!found) {
+				debug_log_message(VERBOSE_DEBUG, DEBUG_AREA_MAIN,
+						"(DBG) skip ACL %s: Application doesn't match", p_acl->aclname);
 				continue;
+			}
 			log_message(VERBOSE_DEBUG, DEBUG_AREA_MAIN,
 				    "[plaintext] App match (%s)",
 				    element->app_name);
@@ -1304,11 +1262,12 @@ G_MODULE_EXPORT GSList *acl_check(connection_t * element, gpointer params)
 
 		/*  We have a match 8-) */
 		log_message(VERBOSE_DEBUG, DEBUG_AREA_MAIN,
-			    "[plaintext] matching with decision %d",
-			    p_acl->decision);
+			    "[plaintext] matching with ACL %s and decision %d",
+			    p_acl->aclname, p_acl->decision);
 		this_acl = g_new0(struct acl_group, 1);
 		g_assert(this_acl);
 		this_acl->answer = p_acl->decision;
+		this_acl->users = g_slist_copy(p_acl->users);
 		this_acl->groups = g_slist_copy(p_acl->groups);
 		if (p_acl->period) {
 			this_acl->period = g_strdup(p_acl->period);
