@@ -2,7 +2,7 @@
  ** Copyright(C) 2003-2006 Eric Leblond <regit@inl.fr>
  ** INL http://www.inl.fr/
  **
- ** $Id: pckt_authsrv.c 3580 2007-08-01 09:43:13Z regit $
+ ** $Id: pckt_authsrv.c 4015 2007-12-04 15:22:47Z regit $
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
@@ -93,8 +93,8 @@ nu_error_t parse_dgram(connection_t * connection, unsigned char *dgram,
 				if (msg_type == AUTH_CONTROL) {
 					connection->state =
 					    AUTH_STATE_DONE;
-					log_user_packet(connection,
-							TCP_STATE_CLOSE);
+					log_message(WARNING, DEBUG_AREA_GW,
+						    "nufw sends non SYN TCP packet, ignoring");
 					free_connection(connection);
 					return NU_EXIT_NO_RETURN;
 				}
@@ -103,8 +103,8 @@ nu_error_t parse_dgram(connection_t * connection, unsigned char *dgram,
 				if (msg_type == AUTH_CONTROL) {
 					connection->state =
 					    AUTH_STATE_DONE;
-					log_user_packet(connection,
-							TCP_STATE_ESTABLISHED);
+					log_message(WARNING, DEBUG_AREA_GW,
+						    "nufw sends SYN ACK TCP packet, ignoring");
 					free_connection(connection);
 					return NU_EXIT_NO_RETURN;
 				}
@@ -291,7 +291,7 @@ nu_error_t authpckt_conntrack(unsigned char *dgram, unsigned int dgram_size)
 	/* Check message content size */
 	if (dgram_size != sizeof(struct nuv4_conntrack_message_t)) {
 		log_message(CRITICAL, DEBUG_AREA_PACKET | DEBUG_AREA_GW,
-				  "Auth conntrack: Improper length of packet (%d instead of %d)",
+				  "Auth conntrack: Improper length of packet (%d instead of %lu)",
 				  dgram_size,
 				  sizeof(struct nuv4_conntrack_message_t));
 		return NU_EXIT_ERROR;
@@ -395,6 +395,7 @@ nu_error_t authpckt_decode(unsigned char **pdgram,
 			if (ret == NU_EXIT_ERROR) {
 				return NU_EXIT_ERROR;
 			}
+			
 			if (ntohs(header->msg_length) < dgram_size) {
 				*pdgram_size =
 				    dgram_size - ntohs(header->msg_length);
@@ -403,7 +404,7 @@ nu_error_t authpckt_decode(unsigned char **pdgram,
 			} else {
 				*pdgram_size = 0;
 			}
-			return NU_EXIT_OK;
+			return ret;
 
 			break;
 		case AUTH_CONN_DESTROY:
@@ -445,7 +446,7 @@ nu_error_t authpckt_decode(unsigned char **pdgram,
 			} else {
 				*pdgram_size = 0;
 			}
-			return NU_EXIT_OK;
+			return ret;
 
 			break;
 		case AUTH_CONN_DESTROY:
