@@ -2,7 +2,7 @@
  ** Copyright(C) 2005-2006 INL
  ** Written by Eric Leblond <regit@inl.fr>
  **
- ** $Id: sasl.c 3496 2007-06-23 14:07:17Z regit $
+ ** $Id: sasl.c 4125 2007-12-18 13:19:03Z haypo $
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
@@ -193,16 +193,20 @@ static int samp_send(gnutls_session session, const char *buffer,
 	unsigned len, alloclen;
 	int result;
 
-	alloclen = ((length / 3) + 1) * 4 + 1;
-	buf = g_alloca(alloclen);
-	result = sasl_encode64(buffer, length, buf + 3, alloclen, &len);
+	/* prefix ("S: ") + base64 length + 1 nul byte */
+	alloclen = 3 + ((length+2)/3)*4 + 1;
+	buf = g_new(char, alloclen);
+	result = sasl_encode64(buffer, length, buf + 3, alloclen - 3, &len);
 	if (result != SASL_OK) {
-		log_message(WARNING, DEBUG_AREA_AUTH, "Encoding data in base64");
+		g_free(buf);
+		log_message(WARNING, DEBUG_AREA_AUTH,
+			"Encoding data in base64 failed");
 		return result;
 	}
 	memcpy(buf, "S: ", 3);
 
 	result = gnutls_record_send(session, buf, len + 3);
+	g_free(buf);
 	return result;
 }
 
@@ -237,7 +241,6 @@ static unsigned samp_recv(gnutls_session session, char *buf, int bufsize)
  */
 
 #define MAX_WAIT_ITER 5
-#define PROTO_WAIT_DELAY 100000
 nu_error_t get_proto_info(user_session_t * c_session)
 {
 	int ret;
@@ -247,7 +250,7 @@ nu_error_t get_proto_info(user_session_t * c_session)
 	/* wait new events during 1 second */
 	FD_ZERO(&wk_set);
 	FD_SET(c_session->socket, &wk_set);
-	tv.tv_sec = 1;
+	tv.tv_sec = nuauthconf->proto_wait_delay;
 	tv.tv_usec = 0;
 	ret = select(c_session->socket + 1, &wk_set, NULL, NULL, &tv);
 	/* catch select() error */
@@ -493,10 +496,8 @@ int sasl_parse_user_os(user_session_t * c_session, char *buf, int buf_size)
 
 	/* check buffer underflow */
 	if (buf_size < (int) sizeof(struct nu_authfield)) {
-		if (inet_ntop
-		    (AF_INET6, &c_session->addr, address,
-		     sizeof(address)) != NULL)
-			g_message("%s sent a too small osfield", address);
+		FORMAT_IPV6(&c_session->addr, address);
+		g_message("%s sent a too small osfield", address);
 		return SASL_FAIL;
 	}
 
@@ -512,12 +513,10 @@ int sasl_parse_user_os(user_session_t * c_session, char *buf, int buf_size)
 
 	dec_buf_size = ntohs(osfield->length);
 	if (dec_buf_size > 1024 || (ntohs(osfield->length) <= 4)) {
-		if (inet_ntop
-		    (AF_INET6, &c_session->addr, address,
-		     sizeof(address)) != NULL)
-			log_message(WARNING, DEBUG_AREA_USER | DEBUG_AREA_AUTH,
-				    "error osfield from %s is uncorrect, announced %d",
-				    address, ntohs(osfield->length));
+		FORMAT_IPV6(&c_session->addr, address);
+		log_message(WARNING, DEBUG_AREA_USER | DEBUG_AREA_AUTH,
+				"error osfield from %s is uncorrect, announced %d",
+				address, ntohs(osfield->length));
 		/* One more gryzor hack */
 		if (dec_buf_size > 4096)
 			log_message(WARNING, DEBUG_AREA_USER | DEBUG_AREA_AUTH,
@@ -552,12 +551,10 @@ int sasl_parse_user_os(user_session_t * c_session, char *buf, int buf_size)
 		if (strlen(os_strings[0]) < 128) {
 			c_session->sysname = string_escape(os_strings[0]);
 			if (c_session->sysname == NULL) {
-				if (inet_ntop
-				    (AF_INET6, &c_session->addr, address,
-				     sizeof(address)) != NULL)
-					log_message(WARNING, DEBUG_AREA_USER | DEBUG_AREA_AUTH,
-						    "received sysname with invalid characters from %s",
-						    address);
+				FORMAT_IPV6(&c_session->addr, address);
+				log_message(WARNING, DEBUG_AREA_USER | DEBUG_AREA_AUTH,
+						"received sysname with invalid characters from %s",
+						address);
 				g_free(dec_buf);
 				return SASL_BADAUTH;
 			}
@@ -567,12 +564,10 @@ int sasl_parse_user_os(user_session_t * c_session, char *buf, int buf_size)
 		if (strlen(os_strings[1]) < 128) {
 			c_session->release = string_escape(os_strings[1]);
 			if (c_session->release == NULL) {
-				if (inet_ntop
-				    (AF_INET6, &c_session->addr, address,
-				     sizeof(address)) != NULL)
-					log_message(WARNING, DEBUG_AREA_USER | DEBUG_AREA_AUTH,
-						    "received release with invalid characters from %s",
-						    address);
+				FORMAT_IPV6(&c_session->addr, address);
+				log_message(WARNING, DEBUG_AREA_USER | DEBUG_AREA_AUTH,
+						"received release with invalid characters from %s",
+						address);
 				g_free(dec_buf);
 				return SASL_BADAUTH;
 			}
@@ -582,12 +577,10 @@ int sasl_parse_user_os(user_session_t * c_session, char *buf, int buf_size)
 		if (strlen(os_strings[2]) < 128) {
 			c_session->version = string_escape(os_strings[2]);
 			if (c_session->version == NULL) {
-				if (inet_ntop
-				    (AF_INET6, &c_session->addr, address,
-				     sizeof(address)) != NULL)
-					log_message(WARNING, DEBUG_AREA_USER | DEBUG_AREA_AUTH,
-						    "received version with invalid characters from %s",
-						    address);
+				FORMAT_IPV6(&c_session->addr, address);
+				log_message(WARNING, DEBUG_AREA_USER | DEBUG_AREA_AUTH,
+						"received version with invalid characters from %s",
+						address);
 				g_free(dec_buf);
 				return SASL_BADAUTH;
 			}
@@ -599,29 +592,23 @@ int sasl_parse_user_os(user_session_t * c_session, char *buf, int buf_size)
 		    c_session->version) {
 
 #ifdef DEBUG_ENABLE
-			if (DEBUG_OR_NOT
-			    (DEBUG_LEVEL_DEBUG, DEBUG_AREA_USER)) {
-				if (inet_ntop
-				    (AF_INET6, &c_session->addr, address,
-				     sizeof(address)) != NULL)
-					g_message
-					    ("user %s at %s uses OS %s ,%s, %s",
-					     c_session->user_name, address,
-					     c_session->sysname,
-					     c_session->release,
-					     c_session->version);
-
+			if (DEBUG_OR_NOT(DEBUG_LEVEL_DEBUG, DEBUG_AREA_USER)) {
+				FORMAT_IPV6(&c_session->addr, address);
+				g_message
+					("user %s at %s uses OS %s ,%s, %s",
+					 c_session->user_name, address,
+					 c_session->sysname,
+					 c_session->release,
+					 c_session->version);
 			}
 #endif
 		}
 		g_strfreev(os_strings);
 	} else {
-		if (inet_ntop
-		    (AF_INET6, &c_session->addr, address,
-		     sizeof(address)) != NULL)
-			log_message(DEBUG, DEBUG_AREA_USER | DEBUG_AREA_AUTH,
-				    "from %s : osfield->option is not OS_SRV ?!",
-				    address);
+		FORMAT_IPV6(&c_session->addr, address);
+		log_message(DEBUG, DEBUG_AREA_USER | DEBUG_AREA_AUTH,
+				"from %s : osfield->option is not OS_SRV ?!",
+				address);
 		g_free(dec_buf);
 		return SASL_FAIL;
 
@@ -879,7 +866,7 @@ int sasl_user_check(user_session_t * c_session)
 	}
 
 	/* format "ip;port" */
-	format_ipv6(&c_session->addr, ipremoteport, sizeof(ipremoteport));
+	FORMAT_IPV6(&c_session->addr, ipremoteport);
 	len = strlen(ipremoteport);
 	secure_snprintf(ipremoteport+len, sizeof(ipremoteport)-len,
 		";%hu", c_session->sport);
@@ -957,14 +944,11 @@ int sasl_user_check(user_session_t * c_session)
 			    "SASL error: authentication process interrupted";
 		}
 		modules_auth_error_log(c_session, err, message);
+		return ret;
 	}
 
 	sasl_dispose(&conn);
 
-	if (ret != SASL_OK)
-		return ret;
-
-
 	/* recv OS datas from client */
 	buf_size = gnutls_record_recv(*(c_session->tls), buf, sizeof buf);
 	if (buf_size <= 0) {
