@@ -3,7 +3,7 @@
  ** Written by  Eric Leblond <regit@inl.fr>
  **             Vincent Deffontaines <gryzor@inl.fr>
  **
- ** $Id: tls_user.c 3427 2007-06-09 21:55:22Z regit $
+ ** $Id: tls_user.c 4126 2007-12-18 13:20:29Z haypo $
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
@@ -35,7 +35,6 @@
  */
 
 extern int nuauth_tls_auth_by_cert;
-struct tls_user_context_t tls_user_context;
 
 /**
  * List of new clients which are in authentication state. This list is
@@ -86,8 +85,8 @@ gboolean remove_socket_from_pre_client_list(int socket)
 }
 
 /**
- * Check pre client list to disconnect connection
- * that are open since too long
+ * Check pre client list to disconnect connections
+ * that have been open for too long
  */
 void* pre_client_check(GMutex *mutex)
 {
@@ -317,10 +316,7 @@ int tls_user_accept(struct tls_user_context_t *context)
 
 	/* Extract client address (convert it to IPv6 if it's IPv4) */
 	if (sockaddr.ss_family == AF_INET) {
-		addr.s6_addr32[0] = 0;
-		addr.s6_addr32[1] = 0;
-		addr.s6_addr32[2] = 0xffff0000;
-		addr.s6_addr32[3] = sockaddr4->sin_addr.s_addr;
+		ipv4_to_ipv6(sockaddr4->sin_addr, &addr);
 		sport = ntohs(sockaddr4->sin_port);
 	} else {
 		addr = sockaddr6->sin6_addr;
@@ -418,10 +414,17 @@ void tls_user_update_mx(struct tls_user_context_t *this)
  *
  * This function has to be called when mutex is locked.
  */
-void tls_user_remove_client(struct tls_user_context_t *this, int sock)
+void tls_user_remove_client(int sock)
 {
-	FD_CLR(sock, &this->tls_rx_set);
-	tls_user_update_mx(this);
+	struct tls_user_context_t *this;
+	GSList *thread_p = nuauthdatas->tls_auth_servers;
+	while (thread_p) {
+		this = ((struct nuauth_thread_t *)thread_p->data)->data;
+		/* search sock among existing select */
+		FD_CLR(sock, &this->tls_rx_set);
+		tls_user_update_mx(this);
+		thread_p = thread_p->next;
+	}
 }
 
 /**
@@ -465,6 +468,7 @@ void tls_user_main_loop(struct tls_user_context_t *context, GMutex * mutex)
 			c_pop = g_async_queue_try_pop(mx_queue);
 		}
 
+		/** \todo carefully check disconnect system */
 		/*
 		 * execute client destruction task
 		 */
@@ -497,6 +501,22 @@ void tls_user_main_loop(struct tls_user_context_t *context, GMutex * mutex)
 				continue;
 			}
 
+			if (errno == EBADF) {
+				/* A client disconnects between FD_SET and select.
+				 * Will try to find it */
+				for (i=0; i<context->mx; ++i){
+					struct stat s;
+					if (FD_ISSET(i, &context->tls_rx_set)){
+						if (fstat(i, &s)<0) {
+							log_message(CRITICAL, DEBUG_AREA_USER,
+								    "Warning: %d is a bad file descriptor.", i);
+							FD_CLR(i, &context->tls_rx_set);
+						}
+					}
+				}
+				continue;
+			}
+
 			log_message(FATAL, DEBUG_AREA_MAIN | DEBUG_AREA_USER,
 				    "select() %s:%d failure: %s",
 				    __FILE__, __LINE__, g_strerror(errno));
@@ -527,120 +547,8 @@ void tls_user_main_loop(struct tls_user_context_t *context, GMutex * mutex)
 	close(context->sck_inet);
 }
 
-/**
- * Bind TLS user socket
- */
-int tls_user_bind(char **errmsg)
+void tls_user_servers_init()
 {
-	struct addrinfo *res;
-	struct addrinfo hints;
-	int ecode;
-	int sck_inet;
-	gint option_value;
-	int result;
-
-	memset(&hints, 0, sizeof hints);
-	hints.ai_flags = AI_PASSIVE;
-	hints.ai_socktype = SOCK_STREAM;
-	hints.ai_family = PF_UNSPEC;
-	ecode =
-	    getaddrinfo(nuauthconf->client_srv, nuauthconf->userpckt_port,
-			&hints, &res);
-	if (ecode != 0) {
-		*errmsg =
-		    g_strdup_printf
-		    ("Invalid clients listening address %s:%s, error: %s",
-		     nuauthconf->client_srv, nuauthconf->userpckt_port,
-		     gai_strerror(ecode));
-		return -1;
-	}
-
-	/* open the socket */
-	if (res->ai_family == PF_INET)
-		log_message(DEBUG, DEBUG_AREA_USER | DEBUG_AREA_MAIN,
-			    "Create user server IPv4 socket");
-	else if (res->ai_family == PF_INET6)
-		log_message(DEBUG, DEBUG_AREA_USER | DEBUG_AREA_MAIN,
-			    "Create user server IPv6 socket");
-
-		log_message(DEBUG, DEBUG_AREA_USER | DEBUG_AREA_MAIN,
-			    "Create user server (any) socket");
-	sck_inet =
-	    socket(res->ai_family, res->ai_socktype, res->ai_protocol);
-	if (sck_inet == -1) {
-		*errmsg = g_strdup("Socket creation failed.");
-		return -1;
-	}
-
-	/* set socket reuse and keep alive option */
-	option_value = 1;
-	setsockopt(sck_inet,
-		   SOL_SOCKET,
-		   SO_REUSEADDR, &option_value, sizeof(option_value));
-	setsockopt(sck_inet,
-		   SOL_SOCKET,
-		   SO_KEEPALIVE, &option_value, sizeof(option_value));
-
-	/* bind */
-	result = bind(sck_inet, res->ai_addr, res->ai_addrlen);
-	if (result < 0) {
-		*errmsg = g_strdup_printf("Unable to bind %s:%s.",
-					  nuauthconf->client_srv,
-					  nuauthconf->userpckt_port);
-		close(sck_inet);
-		return -1;
-	}
-	freeaddrinfo(res);
-	return sck_inet;
-}
-
-/**
- * Create TLS user context.
- */
-int tls_user_init(struct tls_user_context_t *context)
-{
-	confparams_t nuauth_tls_vars[] = {
-		{"nuauth_tls_max_clients", G_TOKEN_INT,
-		 NUAUTH_TLS_MAX_CLIENTS, NULL},
-		{"nuauth_number_authcheckers", G_TOKEN_INT, NB_AUTHCHECK,
-		 NULL},
-		{"nuauth_auth_nego_timeout", G_TOKEN_INT,
-		 AUTH_NEGO_TIMEOUT, NULL}
-	};
-	char *errmsg;
-	int result;
-
-	context->sck_inet = tls_user_bind(&errmsg);
-	if (context->sck_inet < 0) {
-		log_message(FATAL, DEBUG_AREA_MAIN | DEBUG_AREA_USER,
-			    "FATAL ERROR: User bind error: %s", errmsg);
-		log_message(FATAL, DEBUG_AREA_MAIN | DEBUG_AREA_USER,
-			    "Check that nuauth is not running twice. Exit nuauth!");
-		return 0;
-	}
-
-	/* get config file setup */
-	/* parse conf file */
-	parse_conffile(DEFAULT_CONF_FILE,
-		       sizeof(nuauth_tls_vars) / sizeof(confparams_t),
-		       nuauth_tls_vars);
-
-#define READ_CONF(KEY) \
-	get_confvar_value(nuauth_tls_vars, sizeof(nuauth_tls_vars)/sizeof(confparams_t), KEY)
-
-	context->nuauth_tls_max_clients =
-	    *(unsigned int *) READ_CONF("nuauth_tls_max_clients");
-	context->nuauth_number_authcheckers =
-	    *(int *) READ_CONF("nuauth_number_authcheckers");
-	context->nuauth_auth_nego_timeout =
-	    *(int *) READ_CONF("nuauth_auth_nego_timeout");
-#undef READ_CONF
-
-	/* free config struct */
-	free_confparams(nuauth_tls_vars,
-			sizeof(nuauth_tls_vars) / sizeof(confparams_t));
-
-	context->cmd_queue =  g_async_queue_new();
 	/* init sasl stuff */
 	my_sasl_init();
 
@@ -655,9 +563,61 @@ int tls_user_init(struct tls_user_context_t *context)
 	/* create tls sasl worker thread pool */
 	nuauthdatas->tls_sasl_worker =
 	    g_thread_pool_new((GFunc) tls_sasl_connect, NULL,
-			      context->nuauth_number_authcheckers, TRUE,
+			      nuauthconf->nb_auth_checkers, TRUE,
 			      NULL);
 
+	nuauthdatas->user_cmd_queue =  g_async_queue_new();
+}
+
+/**
+ * Create TLS user context.
+ */
+int tls_user_init(struct tls_user_context_t *context)
+{
+	confparams_t nuauth_tls_vars[] = {
+		{"nuauth_tls_max_clients", G_TOKEN_INT,
+		 NUAUTH_TLS_MAX_CLIENTS, NULL},
+		{"nuauth_auth_nego_timeout", G_TOKEN_INT,
+		 AUTH_NEGO_TIMEOUT, NULL}
+	};
+	char *errmsg;
+	int result;
+
+	context->sck_inet = nuauth_bind(&errmsg, context->addr, context->port, "user");
+	if (context->sck_inet < 0) {
+		log_message(FATAL, DEBUG_AREA_MAIN | DEBUG_AREA_USER,
+			    "FATAL ERROR: User bind error: %s", errmsg);
+		log_message(FATAL, DEBUG_AREA_MAIN | DEBUG_AREA_USER,
+			    "Check that nuauth is not running twice. nuauth exiting!");
+		return 0;
+	}
+
+	/* get config file setup */
+	/* parse conf file */
+	if(!parse_conffile(DEFAULT_CONF_FILE,
+		       sizeof(nuauth_tls_vars) / sizeof(confparams_t),
+		       nuauth_tls_vars))
+	{
+	        log_message(FATAL, DEBUG_AREA_MAIN, "Failed to load config file %s", DEFAULT_CONF_FILE);
+		return 0;
+	}
+
+
+#define READ_CONF(KEY) \
+	get_confvar_value(nuauth_tls_vars, sizeof(nuauth_tls_vars)/sizeof(confparams_t), KEY)
+
+	context->nuauth_tls_max_clients =
+	    *(unsigned int *) READ_CONF("nuauth_tls_max_clients");
+	context->nuauth_auth_nego_timeout =
+	    *(int *) READ_CONF("nuauth_auth_nego_timeout");
+#undef READ_CONF
+
+	/* free config struct */
+	free_confparams(nuauth_tls_vars,
+			sizeof(nuauth_tls_vars) / sizeof(confparams_t));
+
+	context->cmd_queue = g_async_queue_new();
+
 	/* listen */
 	result = listen(context->sck_inet, 20);
 	if (result == -1) {
@@ -719,9 +679,7 @@ void *push_worker(GMutex * mutex)
 			/* search in client array */
 			warn_clients(global_msg);
 			/* do we have found something */
-			if (memcmp
-			    (&global_msg->addr, &in6addr_any,
-			     sizeof(in6addr_any)) != 0) {
+			if (!ipv6_equal(&global_msg->addr, &in6addr_any)) {
 				if (global_msg->found == FALSE) {
 					/* if we do ip authentication send request to pool */
 					if (nuauthconf->
@@ -771,17 +729,63 @@ void *push_worker(GMutex * mutex)
  *
  * \return NULL
  */
-void *tls_user_authsrv(GMutex * mutex)
+void *tls_user_authsrv(struct nuauth_thread_t *thread)
 {
-	int ok = tls_user_init(&tls_user_context);
+	struct tls_user_context_t *context = thread->data;
+	int ok = tls_user_init(context);
 	if (ok) {
-		tls_user_main_loop(&tls_user_context, mutex);
+		tls_user_main_loop(context, thread->mutex);
 	} else {
 		nuauth_ask_exit();
 	}
 	return NULL;
 }
 
+void tls_user_start_servers(GSList *servers)
+{
+	char **user_servers;
+	int i = 0;
+	nuauthdatas->tls_auth_servers = NULL;
+
+	tls_user_servers_init();
+
+	/* get raw string from configuration */
+	user_servers = g_strsplit(nuauthconf->client_srv, " ", 0);
+	while (user_servers[i]) {
+		/** \todo free context at program exit */
+		struct tls_user_context_t *context = 
+			g_new0(struct tls_user_context_t, 1);
+		struct nuauth_thread_t *srv_thread =
+			g_new0(struct nuauth_thread_t, 1);
+		char **context_datas = g_strsplit(user_servers[i], ":", 2);
+		if (context_datas[0]) {
+			context->addr = g_strdup(context_datas[0]);
+		} else {
+			log_message(FATAL, DEBUG_AREA_MAIN | DEBUG_AREA_GW,
+			    "Address parsing error at %s:%d (%s)", __FILE__,
+			    __LINE__, user_servers[i]);
+			nuauth_ask_exit();
+		}
+		if (context_datas[1]) {
+			context->port = g_strdup(context_datas[1]);
+		} else {
+			context->port = g_strdup(nuauthconf->userpckt_port);
+		}
+		g_strfreev(context_datas);
+		log_message(INFO, DEBUG_AREA_MAIN | DEBUG_AREA_USER,
+			    "Creating user socket %s:%s",context->addr, context->port);
+
+		thread_new_wdata(srv_thread,
+				 "tls auth server",
+				 (gpointer) context,
+				 tls_user_authsrv);
+		/* Append newly created server to list */
+		nuauthdatas->tls_auth_servers = g_slist_prepend(nuauthdatas->tls_auth_servers,
+								srv_thread);
+		i++;
+	}
+	g_strfreev(user_servers);
+}
 
 /**
  * @}
