@@ -5,7 +5,7 @@
 **            Vincent Deffontaines <vincent@gryzor.com>
 ** INL http://www.inl.fr/
 **
-** $Id: common.c 3562 2007-07-25 22:58:40Z regit $
+** $Id: common.c 4183 2007-12-21 15:30:26Z regit $
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -118,9 +118,8 @@ void psuppress(packet_idl * previous, packet_idl * current)
 unsigned long padd(packet_idl * current)
 {
 	if (track_size <= packets_list.length) {
-		log_area_printf(DEBUG_AREA_PACKET, DEBUG_LEVEL_MESSAGE,
-				"Warning: queue is full, drop element %d",
-				current->id);
+		log_area_printf(DEBUG_AREA_PACKET, DEBUG_LEVEL_WARNING,
+				"Warning: queue is full, dropping element");
 		IPQ_SET_VERDICT(current->id, NF_DROP);
 		return 0;
 	}
