@@ -4,7 +4,7 @@
  **            Vincent Deffontaines <vincent@gryzor.com>
  ** INL http://www.inl.fr/
  **
- ** $Id: main.c 3435 2007-06-12 20:49:25Z regit $
+ ** $Id: main.c 4210 2008-01-02 17:13:51Z toady $
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
@@ -369,6 +369,31 @@ void create_nuauth_address()
 }
 
 
+/**
+ * Initialization checks
+ *  - check key and cert files
+ */
+int init_checks()
+{
+#if USE_X509
+	if (!init_x509_filenames()) {
+		printf("ERROR: Unable to allocate memory for "
+				"key or cert filename!\n");
+		return 0;
+	}
+	if (access(key_file, R_OK)) {
+		printf("ERROR: Unable to read key file: %s\n", key_file);
+		return 0;
+	}
+	if (access(cert_file, R_OK)) {
+		printf("ERROR: Unable to read key file: %s\n", cert_file);
+		return 0;
+	}
+#endif
+	return 1;
+}
+
+
 /**
  * Main function of NuFW:
  *   - Initialize variables
@@ -398,7 +423,7 @@ int main(int argc, char *argv[])
 {
 	/* option */
 #if USE_NFQUEUE
-	char *options_list = "DhVvmq:"
+	char *options_list = "SDhVvmq:"
 #ifdef HAVE_NFQ_SET_QUEUE_MAXLEN
 	    "L:"
 #endif
@@ -408,7 +433,7 @@ int main(int argc, char *argv[])
 #endif
 	    ;
 #else
-	char *options_list = "DhVvmc:k:a:n:d:p:t:T:A:";
+	char *options_list = "SDhVvmc:k:a:n:d:p:t:T:A:";
 #endif
 	int option, daemonize = 0;
 	char *version = PACKAGE_VERSION;
@@ -438,6 +463,7 @@ int main(int argc, char *argv[])
 #endif
 #endif
 	nufw_set_mark = 0;
+	nufw_strict_tls = 0;
 
 
 	log_area_printf(DEBUG_AREA_MAIN, DEBUG_LEVEL_VERBOSE_DEBUG,
@@ -514,6 +540,9 @@ int main(int argc, char *argv[])
 		case 'm':
 			nufw_set_mark = 1;
 			break;
+		case 'S':
+			nufw_strict_tls = 1;
+			break;
 #if USE_NFQUEUE
 		case 'q':
 			sscanf(optarg, "%hu", &nfqueue_num);
@@ -553,13 +582,14 @@ int main(int argc, char *argv[])
 \t-h: display this help and exit\n\
 \t-V: display version and exit\n\
 \t-D: daemonize\n\
+\t-S: do strict checking of TLS negociation\n\
 \t-k: use specified file as key file\n\
 \t-c: use specified file as cert file\n\
 \t-a: use specified file as ca file (strict checking is done if selected) (default: none)\n\
 \t-n: use specified string as the needed DN of nuauth (inforce certificate checking) (default: none)\n\
 \t-v: increase debug level (+1 for each 'v') (max useful number: 10)\n\
 \t-A: debug areas (see man page for details)\n\
-\t-m: mark packet with userid (needed for connection expiration)\n"
+\t-m: mark packet with nuauth provided mark\n"
 #ifdef HAVE_LIBCONNTRACK
 				"\t-C: listen to conntrack events (needed for connection expiration)\n\
 \t-M: only report event on marked connections to nuauth (implies -C and -m)\n"
@@ -585,6 +615,10 @@ int main(int argc, char *argv[])
 		exit(EXIT_FAILURE);
 	}
 
+	if (!init_checks()) {
+		exit(EXIT_FAILURE);
+	}
+
 	/* Daemon code */
 	if (daemonize == 1) {
 		nufw_daemonize();
@@ -637,6 +671,11 @@ int main(int argc, char *argv[])
 	log_area_printf(DEBUG_AREA_MAIN, DEBUG_LEVEL_FATAL,
 			"[+] NuFW " VERSION " started");
 
+	if ( ! nufw_strict_tls ) {
+		log_area_printf(DEBUG_AREA_MAIN, DEBUG_LEVEL_WARNING,
+				"NuFW did not start under the TLS strict mode");
+	}
+
 	/* control stuff */
 	pckt_tx = pckt_rx = 0;
 	while (1 == 1) {
