@@ -4,7 +4,7 @@
  **            Vincent Deffontaines <vincent@gryzor.com>
  ** INL http://www.inl.fr/
  **
- ** $Id: nufw.h 3077 2007-03-28 08:57:44Z regit $
+ ** $Id: nufw.h 4056 2007-12-08 11:07:02Z regit $
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
@@ -59,6 +59,9 @@
 #include <stdio.h>
 #include <string.h>
 #include <signal.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <unistd.h>
 #include <assert.h>
 #include <strings.h>
 #include <gnutls/gnutls.h>
@@ -153,6 +156,7 @@ struct queued_pckt {
 
 struct nuauth_conn tls;
 
+int init_x509_filenames();
 gnutls_session *tls_connect();
 pthread_cond_t *session_destroyed_cond;
 pthread_cond_t *session_active_cond;
