@@ -1,10 +1,10 @@
 /*
- ** Copyright (C) 2002-2006 INL
+ ** Copyright (C) 2002-2007 INL
  ** Written by Eric Leblond <eric@regit.org>
  **            Vincent Deffontaines <vincent@gryzor.com>
  ** INL http://www.inl.fr/
  **
- ** $Id: packetsrv.c 3535 2007-07-01 22:13:26Z regit $
+ ** $Id: packetsrv.c 4242 2008-01-07 09:25:28Z regit $
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
@@ -209,11 +209,8 @@ int packetsrv_open(void *data)
 	}
 
 	/* unbinding existing nf_queue handler for AF_INET (if any) */
-	if (nfq_unbind_pf(h, AF_INET) < 0) {
-		log_area_printf(DEBUG_AREA_MAIN, DEBUG_LEVEL_CRITICAL,
-				"[!] Error during nfq_unbind_pf()");
-		return -1;
-	}
+	/* ignoring return, see http://www.spinics.net/lists/netfilter/msg42063.html */
+	nfq_unbind_pf(h, AF_INET);
 
 	/* binding nfnetlink_queue as nf_queue handler for AF_INET */
 	if (nfq_bind_pf(h, AF_INET) < 0) {
@@ -223,11 +220,7 @@ int packetsrv_open(void *data)
 	}
 
 	/* unbinding existing nf_queue handler for AF_INET6 (if any) */
-	if (nfq_unbind_pf(h, AF_INET6) < 0) {
-		log_area_printf(DEBUG_AREA_MAIN, DEBUG_LEVEL_CRITICAL,
-				"[!] Error during nfq_unbind_pf()");
-		return -1;
-	}
+	nfq_unbind_pf(h, AF_INET6);
 
 	/* binding nfnetlink_queue as nf_queue handler for AF_INET6 */
 	if (nfq_bind_pf(h, AF_INET6) < 0) {
@@ -382,6 +375,7 @@ void *packetsrv(void *void_arg)
 #endif
 	int rv;
 	int select_result;
+	int max_fd;
 	fd_set wk_set;
 
 #ifdef HAVE_NLIF_CATCH
@@ -420,13 +414,49 @@ void *packetsrv(void *void_arg)
 		FD_SET(fd, &wk_set);
 #ifdef HAVE_NLIF_CATCH
 		FD_SET(if_fd, &wk_set);
+
+		if (fd >= if_fd) {
+			max_fd = fd + 1;
+		} else {
+			max_fd = if_fd + 1;
+		}
+#else
+		max_fd = fd + 1;
 #endif
-		select_result = select(fd + 1, &wk_set, NULL, NULL, &tv);
+
+		select_result = select(max_fd, &wk_set, NULL, NULL, &tv);
 		if (select_result == -1) {
 			int err = errno;
 			if (err == EINTR) {
 				continue;
 			}
+
+			if (err == EBADF) {
+				struct stat s;
+#ifdef HAVE_NLIF_CATCH
+				if ((fstat(if_fd, &s)<0)) {
+					iface_table_close(nlif_handle);
+
+					nlif_handle = iface_table_open();
+					if (!nlif_handle)
+						exit(EXIT_FAILURE);
+
+					if_fd = nlif_fd(nlif_handle);
+					if (if_fd < 0) {
+						exit(EXIT_FAILURE);
+					}
+				}
+#endif
+				if ((fstat(fd, &s)<0)) {
+					packetsrv_close(0);
+#ifdef HAVE_NLIF_CATCH
+					fd = packetsrv_open(nlif_handle);
+#else
+					fd = packetsrv_open(NULL);
+#endif
+				}
+				continue;
+			}
 			log_area_printf(DEBUG_AREA_MAIN,
 					DEBUG_LEVEL_CRITICAL,
 					"[!] FATAL ERROR: Error of select() in netfilter queue thread (code %i)!",
@@ -684,7 +714,7 @@ int auth_request_send(uint8_t type, struct queued_pckt *pckt_datas)
 			tls.auth_server_running = 1;
 		} else {
 			log_area_printf(DEBUG_AREA_GW,
-					DEBUG_LEVEL_WARNING,
+					DEBUG_LEVEL_CRITICAL,
 					"[!] TLS connection to nuauth can NOT be restored");
 			return 0;
 		}
