@@ -4,7 +4,7 @@
  **            Vincent Deffontaines <vincent@gryzor.com>
  ** INL http://www.inl.fr/
  **
- ** $Id: tls.c 3347 2007-05-21 13:19:22Z haypo $
+ ** $Id: tls.c 4277 2008-01-08 17:24:47Z lds $
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
@@ -22,8 +22,13 @@
 
 
 #include "nufw.h"
+#include <time.h>
 #include <gnutls/x509.h>
 
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <unistd.h>
+
 /**
  * \file nufw/tls.c
  * \brief Create a TLS connection to NuAuth
@@ -31,6 +36,88 @@
  * Create a TLS connection to NuAuth using tls_connect().
  */
 
+static unsigned int tls_cert_verify(gnutls_session *tls_session, char *nuauth_cert_dn);
+static unsigned int check_nuauth_cert_dn(gnutls_session *tls_session);
+extern int init_x509_filenames(void);
+extern gnutls_session *tls_connect(void);
+
+
+/**
+ * Perform certificate checks: Logs for each check and 
+ * return the global check result.
+ *
+ * Returns 1 on error, 0 on success.
+ */
+unsigned int tls_cert_verify(gnutls_session *tls_session, char *nuauth_cert_dn)
+{
+
+	int ret;
+	unsigned int status = 0;
+	time_t now;
+
+	/* we need to verify received certificates */
+	ret = gnutls_certificate_verify_peers2(*tls_session, &status);
+	if (ret < 0) {
+		log_area_printf(DEBUG_AREA_GW,
+				DEBUG_LEVEL_WARNING,
+				"TLS: Certificate authority verification failed: %s",
+				gnutls_strerror(ret));
+		return 1;
+	}
+	if (status) {
+		char buffer[200];
+
+		buffer[0] = 0;
+		if (status & GNUTLS_CERT_INVALID) 
+			SECURE_STRNCAT(buffer, " invalid.", sizeof(buffer));
+		if (status & GNUTLS_CERT_REVOKED)
+			SECURE_STRNCAT(buffer, " revoked.", sizeof(buffer));
+		if (status & GNUTLS_CERT_SIGNER_NOT_FOUND)
+			SECURE_STRNCAT(buffer, " signer not found.", sizeof(buffer));
+		if (status & GNUTLS_CERT_SIGNER_NOT_CA)
+			SECURE_STRNCAT(buffer, " signer not a CA.", sizeof(buffer));
+#ifdef GNUTLS_CERT_INSECURE_ALGORITHM
+		if (status & GNUTLS_CERT_INSECURE_ALGORITHM)
+			SECURE_STRNCAT(buffer, " use of insecure algorithm.", sizeof(buffer));
+#endif
+
+		log_area_printf(DEBUG_AREA_GW, DEBUG_LEVEL_WARNING,
+				"TLS: Invalid certificates received from nuauth server:%s",
+				buffer);
+		return 1;
+	}
+
+	now = time(NULL);
+	if (gnutls_certificate_activation_time_peers(*tls_session) > now) {
+		char buffer[200];
+		buffer[0] = 0;
+
+		SECURE_STRNCAT(buffer, " server certificate not yet activated", sizeof(buffer));
+		log_area_printf(DEBUG_AREA_GW, DEBUG_LEVEL_WARNING,
+				"TLS: Invalid certificates received from nuauth server:%s",
+				buffer);
+		return 1;    
+	}
+	if (gnutls_certificate_expiration_time_peers(*tls_session) < now) {
+		char buffer[200];
+		buffer[0] = 0;
+
+		SECURE_STRNCAT(buffer, " server certificate expired", sizeof(buffer));
+		log_area_printf(DEBUG_AREA_GW, DEBUG_LEVEL_WARNING,
+				"TLS: Invalid certificates received from nuauth server:%s",
+				buffer);
+		return 1;    
+	}
+
+	if (nuauth_cert_dn) {
+		if (!check_nuauth_cert_dn(tls_session)) 
+			return 1;
+	}
+
+
+	return 0;
+}
+
 /**
  * Check nuauth certification domain name (DN).
  *
@@ -43,11 +130,6 @@ unsigned int check_nuauth_cert_dn(gnutls_session *tls_session)
 	size_t size;
 	int ret;
 
-#if 0
-	unsigned int algo, bits;
-	time_t expiration_time,
-	       activation_time;
-#endif
 	const gnutls_datum *cert_list;
 	unsigned int cert_list_size = 0;
 	gnutls_x509_crt cert;
@@ -80,14 +162,6 @@ unsigned int check_nuauth_cert_dn(gnutls_session *tls_session)
 		return 0;
 	}
 
-	/* TODO: verify date */
-#if 0
-	expiration_time = gnutls_x509_crt_get_expiration_time(cert);
-	activation_time = gnutls_x509_crt_get_activation_time(cert);
-
-	/* Extract some of the public key algorithm's parameters */
-	algo = gnutls_x509_crt_get_pk_algorithm(cert, &bits);
-#endif
 	size = sizeof(dn);
 	ret = gnutls_x509_crt_get_dn(cert, dn, &size);
 	if (ret != 0) {
@@ -103,6 +177,57 @@ unsigned int check_nuauth_cert_dn(gnutls_session *tls_session)
 			 dn);
 		return 0;
 	}
+
+
+	return 1;
+}
+
+
+/**
+ * Inialialize key_file and cert_file variables
+ */
+int init_x509_filenames()
+{
+#if USE_X509
+	if (!key_file) {
+		key_file =
+			(char *) calloc(strlen(CONFIG_DIR) + strlen(KEYFILE) +
+					2, sizeof(char));
+		if (!key_file) {
+			return 0;
+		}
+		strcat(key_file, CONFIG_DIR);
+		strcat(key_file, "/");
+		strcat(key_file, KEYFILE);
+	}
+	if (!cert_file) {
+		cert_file =
+			(char *) calloc(strlen(CONFIG_DIR) + strlen(CERTFILE) +
+					2, sizeof(char));
+		if (!cert_file) {
+			return 0;
+		}
+		strcat(cert_file, CONFIG_DIR);
+		strcat(cert_file, "/");
+		strcat(cert_file, CERTFILE);
+	}
+#endif
+	return 1;
+}
+
+int check_key_perms(const char* filename)
+{
+	struct stat infos;
+
+	if (stat(filename, &infos) != 0) {
+		return 0;
+	}
+	
+	/* Check it's owned by the current user */
+	if (infos.st_mode & S_IRGRP || infos.st_mode & S_IROTH) {
+		return 0;
+	}
+	
 	return 1;
 }
 
@@ -123,31 +248,10 @@ gnutls_session *tls_connect()
 	const int cert_type_priority[3] = { GNUTLS_CRT_X509, 0 };
 
 	/* compute patch key_file */
-	if (!key_file) {
-		key_file =
-		    (char *) calloc(strlen(CONFIG_DIR) + strlen(KEYFILE) +
-				    2, sizeof(char));
-		if (!key_file) {
-			log_area_printf(DEBUG_AREA_MAIN, DEBUG_LEVEL_DEBUG,
-					"Couldn't malloc for key_file!");
-			return NULL;
-		}
-		strcat(key_file, CONFIG_DIR);
-		strcat(key_file, "/");
-		strcat(key_file, KEYFILE);
-	}
-	if (!cert_file) {
-		cert_file =
-		    (char *) calloc(strlen(CONFIG_DIR) + strlen(CERTFILE) +
-				    2, sizeof(char));
-		if (!cert_file) {
-			log_area_printf(DEBUG_AREA_MAIN, DEBUG_LEVEL_DEBUG,
-					"Couldn't malloc for cert_file!");
-			return NULL;
-		}
-		strcat(cert_file, CONFIG_DIR);
-		strcat(cert_file, "/");
-		strcat(cert_file, CERTFILE);
+	if (!init_x509_filenames()) {
+		log_area_printf(DEBUG_AREA_MAIN, DEBUG_LEVEL_DEBUG,
+				"Couldn't malloc for key or cert filename!");
+		return NULL;
 	}
 
 	/* test if key exists */
@@ -157,6 +261,7 @@ gnutls_session *tls_connect()
 				key_file);
 		return NULL;
 	}
+
 	if (access(cert_file, R_OK)) {
 		log_area_printf(DEBUG_AREA_GW, DEBUG_LEVEL_WARNING,
 				"TLS: can not access cert file \"%s\"!",
@@ -187,6 +292,16 @@ gnutls_session *tls_connect()
 			return NULL;
 		}
 	}
+
+	if (!check_key_perms(key_file)) {
+		log_area_printf(DEBUG_AREA_MAIN,
+				DEBUG_LEVEL_FATAL,
+				"Private key have too open permissions");
+		if (ca_file || nufw_strict_tls) {
+			return NULL;
+		}
+	}
+
 	ret =
 	    gnutls_certificate_set_x509_key_file(tls.xcred, cert_file,
 						 key_file,
@@ -198,6 +313,7 @@ gnutls_session *tls_connect()
 		return NULL;
 
 	}
+
 #endif
 
 	/* Initialize TLS session */
@@ -221,8 +337,10 @@ gnutls_session *tls_connect()
 	/* connect */
 	if (tls_socket <= 0)
 		return NULL;
+
 	if (connect(tls_socket, adr_srv->ai_addr, adr_srv->ai_addrlen) ==
 	    -1) {
+		close(tls_socket);
 		return NULL;
 	}
 
@@ -276,39 +394,20 @@ gnutls_session *tls_connect()
 		return NULL;
 	}
 #if USE_X509
-	if (ca_file) {
-		unsigned int status = 0;
-		/* we need to verify received certificates */
-		ret = gnutls_certificate_verify_peers2(*tls_session, &status);
-		if (ret < 0) {
-			log_area_printf(DEBUG_AREA_GW,
-					DEBUG_LEVEL_WARNING,
-					"TLS: Certificate authority verification failed: %s",
-					gnutls_strerror(ret));
+
+	ret = tls_cert_verify(tls_session, nuauth_cert_dn);
+	if (ca_file || nufw_strict_tls) {
+		if (ret) {
+			log_area_printf(DEBUG_AREA_GW, DEBUG_LEVEL_FATAL,
+					"TLS: Either NuFW is running under the TLS strict mode, either you provided the CA file as parameter. Cannot authenticate the packet.");
 			return NULL;
 		}
-		if (status) {
-			char buffer[200];
-			buffer[0] = 0;
-			if (status & GNUTLS_CERT_INVALID)
-				SECURE_STRNCAT(buffer, " invalid", sizeof(buffer));
-			if (status & GNUTLS_CERT_REVOKED)
-				SECURE_STRNCAT(buffer, ", revoked", sizeof(buffer));
-			if (status & GNUTLS_CERT_SIGNER_NOT_FOUND)
-				SECURE_STRNCAT(buffer, ", signer not found", sizeof(buffer));
-			if (status & GNUTLS_CERT_SIGNER_NOT_CA)
-				SECURE_STRNCAT(buffer, ", signer not a CA", sizeof(buffer));
-			log_area_printf(DEBUG_AREA_GW, DEBUG_LEVEL_WARNING,
-					"TLS: Invalid certificates received from nuauth server:%s",
-					buffer);
-			return NULL;
-		}
-		if (nuauth_cert_dn) {
-			if (!check_nuauth_cert_dn(tls_session)) {
-				return NULL;
-			}
-		}
 	}
+
+	if ( ! nufw_strict_tls )
+			log_area_printf(DEBUG_AREA_GW, DEBUG_LEVEL_WARNING,
+					"TLS: The strict mode will be activated by default in next major stable release.");
+
 #endif
 	return tls_session;
 }
